@@ -5,19 +5,19 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-from mercurial import error, hg, merge as mergemod
+from mercurial import error
 
 from tortoisehg.util import hglib
 from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt import qtlib, csinfo, i18n, cmdui, status, resolve
-from tortoisehg.hgqt import qscilib, thgrepo, messageentry
+from tortoisehg.hgqt import qscilib, thgrepo, messageentry, wctxcleaner
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 
 class BackoutDialog(QWizard):
 
-    def __init__(self, rev, repo, parent):
+    def __init__(self, repoagent, rev, parent=None):
         super(BackoutDialog, self).__init__(parent)
         f = self.windowFlags()
         self.setWindowFlags(f & ~Qt.WindowContextHelpButtonHint)
@@ -26,22 +26,23 @@ class BackoutDialog(QWizard):
         self.parentbackout = False
         self.backoutmergeparentrev = None
 
+        repo = repoagent.rawRepo()
         self.setWindowTitle(_('Backout - %s') % repo.displayname)
         self.setWindowIcon(qtlib.geticon('hg-revert'))
         self.setOption(QWizard.NoBackButtonOnStartPage, True)
         self.setOption(QWizard.NoBackButtonOnLastPage, True)
         self.setOption(QWizard.IndependentPages, True)
 
-        self.addPage(SummaryPage(repo, self))
-        self.addPage(BackoutPage(repo, self))
-        self.addPage(CommitPage(repo, self))
-        self.addPage(ResultPage(repo, self))
+        self.addPage(SummaryPage(repoagent, self))
+        self.addPage(BackoutPage(repoagent, self))
+        self.addPage(CommitPage(repoagent, self))
+        self.addPage(ResultPage(repoagent, self))
         self.currentIdChanged.connect(self.pageChanged)
 
         self.resize(QSize(700, 489).expandedTo(self.minimumSizeHint()))
 
-        repo.repositoryChanged.connect(self.repositoryChanged)
-        repo.configChanged.connect(self.configChanged)
+        repoagent.repositoryChanged.connect(self.repositoryChanged)
+        repoagent.configChanged.connect(self.configChanged)
 
     @pyqtSlot()
     def repositoryChanged(self):
@@ -61,9 +62,13 @@ class BackoutDialog(QWizard):
 
 
 class BasePage(QWizardPage):
-    def __init__(self, repo, parent):
+    def __init__(self, repoagent, parent):
         super(BasePage, self).__init__(parent)
-        self.repo = repo
+        self._repoagent = repoagent
+
+    @property
+    def repo(self):
+        return self._repoagent.rawRepo()
 
     def validatePage(self):
         'user pressed NEXT button, can we proceed?'
@@ -90,10 +95,11 @@ class BasePage(QWizardPage):
 
 class SummaryPage(BasePage):
 
-    def __init__(self, repo, parent):
-        super(SummaryPage, self).__init__(repo, parent)
-        self.clean = False
-        self.th = None
+    def __init__(self, repoagent, parent):
+        super(SummaryPage, self).__init__(repoagent, parent)
+        self._wctxcleaner = wctxcleaner.WctxCleaner(repoagent, self)
+        self._wctxcleaner.checkStarted.connect(self._onCheckStarted)
+        self._wctxcleaner.checkFinished.connect(self._onCheckFinished)
 
     def initializePage(self):
         if self.layout():
@@ -221,7 +227,7 @@ class SummaryPage(BasePage):
                  'or <a href="discard"><b>discard</b></a> changes.')
         wd_text = QLabel(text)
         wd_text.setWordWrap(True)
-        wd_text.linkActivated.connect(self.onLinkActivated)
+        wd_text.linkActivated.connect(self._wctxcleaner.runCleaner)
         self.wd_text = wd_text
         self.groups.add(wd_text, 'dirty')
         self.layout().addWidget(wd_text)
@@ -238,7 +244,7 @@ class SummaryPage(BasePage):
 
     def isComplete(self):
         'should Next button be sensitive?'
-        return self.clean
+        return self._wctxcleaner.isClean()
 
     def repositoryChanged(self):
         'repository has detected a change to changelog or parents'
@@ -248,31 +254,26 @@ class SummaryPage(BasePage):
 
     def canExit(self):
         'can backout tool be closed?'
-        if self.th is not None and self.th.isRunning():
-            self.th.cancel()
-            self.th.wait()
+        if self._wctxcleaner.isChecking():
+            self._wctxcleaner.cancelCheck()
         return True
 
     def currentPage(self):
         self.refresh()
 
     def refresh(self):
-        if self.th is None:
-            self.th = CheckThread(self.repo, self)
-            self.th.finished.connect(self.threadFinished)
-        if self.th.isRunning():
-            return
-        self.groups.set_visible(True, 'prog')
-        self.th.start()
+        self._wctxcleaner.check()
 
     @pyqtSlot()
-    def threadFinished(self):
+    def _onCheckStarted(self):
+        self.groups.set_visible(True, 'prog')
+
+    @pyqtSlot(bool)
+    def _onCheckFinished(self, clean):
         self.groups.set_visible(False, 'prog')
-        if self.th.canceled:
+        if self._wctxcleaner.isCheckCanceled():
             return
-        dirty, parents = self.th.results
-        self.clean = not dirty
-        if dirty:
+        if not clean:
             self.groups.set_visible(True, 'dirty')
             self.wd_status.set_status(_('<b>Uncommitted local changes '
                                         'are detected</b>'), 'thg-warning')
@@ -281,50 +282,10 @@ class SummaryPage(BasePage):
             self.wd_status.set_status(_('Clean'), True)
         self.completeChanged.emit()
 
-    @pyqtSlot(QString)
-    def onLinkActivated(self, cmd):
-        cmd = hglib.fromunicode(cmd)
-        repo = self.repo
-        if cmd == 'commit':
-            from tortoisehg.hgqt import commit
-            dlg = commit.CommitDialog(repo, [], {}, self.wizard())
-            dlg.finished.connect(dlg.deleteLater)
-            dlg.exec_()
-            self.refresh()
-        elif cmd == 'shelve':
-            from tortoisehg.hgqt import shelve
-            dlg = shelve.ShelveDialog(repo, self.wizard())
-            dlg.finished.connect(dlg.deleteLater)
-            dlg.exec_()
-            self.refresh()
-        elif cmd.startswith('discard'):
-            if cmd != 'discard:noconfirm':
-                labels = [(QMessageBox.Yes, _('&Discard')),
-                          (QMessageBox.No, _('Cancel'))]
-                if not qtlib.QuestionMsgBox(_('Confirm Discard'),
-                         _('Discard outstanding changes to working directory?'),
-                         labels=labels, parent=self):
-                    return
-            def finished(ret):
-                repo.decrementBusyCount()
-                self.refresh()
-            cmdline = ['update', '--clean', '--repository', repo.root,
-                       '--rev', '.']
-            self.runner = cmdui.Runner(True, self)
-            self.runner.commandFinished.connect(finished)
-            repo.incrementBusyCount()
-            self.runner.run(cmdline)
-        elif cmd == 'view':
-            dlg = status.StatusDialog(repo, [], {}, self)
-            dlg.exec_()
-            self.refresh()
-        else:
-            raise 'unknown command: %s' % cmd
-
 
 class BackoutPage(BasePage):
-    def __init__(self, repo, parent):
-        super(BackoutPage, self).__init__(repo, parent)
+    def __init__(self, repoagent, parent):
+        super(BackoutPage, self).__init__(repoagent, parent)
         self.backoutcomplete = False
 
         self.setTitle(_('Backing out, then merging...'))
@@ -398,7 +359,7 @@ class BackoutPage(BasePage):
     @pyqtSlot(QString)
     def onLinkActivated(self, cmd):
         if cmd == 'resolve':
-            dlg = resolve.ResolveDialog(self.repo, self)
+            dlg = resolve.ResolveDialog(self._repoagent, self)
             dlg.finished.connect(dlg.deleteLater)
             dlg.exec_()
             if self.autonext.isChecked():
@@ -408,8 +369,8 @@ class BackoutPage(BasePage):
 
 class CommitPage(BasePage):
 
-    def __init__(self, repo, parent):
-        super(CommitPage, self).__init__(repo, parent)
+    def __init__(self, repoagent, parent):
+        super(CommitPage, self).__init__(repoagent, parent)
         self.commitComplete = False
 
         self.setTitle(_('Commit backout and merge results'))
@@ -417,6 +378,8 @@ class CommitPage(BasePage):
         self.setLayout(QVBoxLayout())
         self.setCommitPage(True)
 
+        repo = repoagent.rawRepo()
+
         # csinfo
         def label_func(widget, item, ctx):
             if item == 'rev':
@@ -564,7 +527,7 @@ class CommitPage(BasePage):
     @pyqtSlot(QString)
     def onLinkActivated(self, cmd):
         if cmd == 'view':
-            dlg = status.StatusDialog(self.repo, [], {}, self)
+            dlg = status.StatusDialog(self._repoagent, [], {}, self)
             dlg.exec_()
             self.refresh()
 
@@ -618,8 +581,8 @@ class CommitPage(BasePage):
 
 
 class ResultPage(BasePage):
-    def __init__(self, repo, parent):
-        super(ResultPage, self).__init__(repo, parent)
+    def __init__(self, repoagent, parent):
+        super(ResultPage, self).__init__(repoagent, parent)
         self.setTitle(_('Finished'))
         self.setSubTitle(' ')
         self.setFinalPage(True)
@@ -635,27 +598,3 @@ class ResultPage(BasePage):
     def currentPage(self):
         self.bkCsInfo.update(self.repo['tip'])
         self.wizard().setOption(QWizard.NoCancelButton, True)
-
-
-class CheckThread(QThread):
-    def __init__(self, repo, parent):
-        QThread.__init__(self, parent)
-        self.repo = hg.repository(repo.ui, repo.root)
-        self.results = (False, 1)
-        self.canceled = False
-
-    def run(self):
-        self.repo.dirstate.invalidate()
-        unresolved = False
-        for root, path, status in thgrepo.recursiveMergeStatus(self.repo):
-            if self.canceled:
-                return
-            if status == 'u':
-                unresolved = True
-                break
-        wctx = self.repo[None]
-        dirty = bool(wctx.dirty()) or unresolved
-        self.results = (dirty, len(wctx.parents()))
-
-    def cancel(self):
-        self.canceled = True
