@@ -8,24 +8,21 @@
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
 
-import os
+from mercurial import revset
 
-from mercurial import revset, merge as mergemod
-
-from tortoisehg.util import hglib
 from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import qtlib, csinfo, cmdui, commit, thgrepo
+from tortoisehg.hgqt import csinfo, cmdui, commit, wctxcleaner
 
 BB = QDialogButtonBox
 
 class CompressDialog(QDialog):
     showMessage = pyqtSignal(QString)
 
-    def __init__(self, repo, revs, parent):
+    def __init__(self, repoagent, revs, parent):
         super(CompressDialog, self).__init__(parent)
         f = self.windowFlags()
         self.setWindowFlags(f & ~Qt.WindowContextHelpButtonHint)
-        self.repo = repo
+        self._repoagent = repoagent
         self.revs = revs
 
         box = QVBoxLayout()
@@ -54,7 +51,6 @@ class CompressDialog(QDialog):
         self.cmd.commandFinished.connect(self.commandFinished)
         self.cmd.setShowOutput(True)
         self.showMessage.connect(self.cmd.stbar.showMessage)
-        self.cmd.stbar.linkActivated.connect(self.linkActivated)
         self.layout().addWidget(self.cmd, 2)
 
         bbox = QDialogButtonBox()
@@ -67,51 +63,40 @@ class CompressDialog(QDialog):
         self.bbox = bbox
 
         self.showMessage.emit(_('Checking...'))
-        QTimer.singleShot(0, self.checkStatus)
+        self._wctxcleaner = wctxcleaner.WctxCleaner(repoagent, self)
+        self._wctxcleaner.checkFinished.connect(self._checkCompleted)
+        self.cmd.stbar.linkActivated.connect(self._wctxcleaner.runCleaner)
+        QTimer.singleShot(0, self._wctxcleaner.check)
 
         self.setMinimumWidth(480)
         self.setMaximumHeight(800)
         self.resize(0, 340)
+        repo = repoagent.rawRepo()
         self.setWindowTitle(_('Compress - %s') % repo.displayname)
 
-    def checkStatus(self):
-        repo = self.repo
-        class CheckThread(QThread):
-            def __init__(self, parent):
-                QThread.__init__(self, parent)
-                self.dirty = False
+        self.restoreSettings()
 
-            def run(self):
-                wctx = repo[None]
-                if len(wctx.parents()) > 1:
-                    self.dirty = True
-                elif wctx.dirty():
-                    self.dirty = True
-                else:
-                    for root, path, status in thgrepo.recursiveMergeStatus(repo):
-                        if status == 'u':
-                            self.dirty = True
-                            break
-        def completed():
-            self.th.wait()
-            if self.th.dirty:
-                self.compressbtn.setEnabled(False)
-                txt = _('Before compress, you must <a href="commit">'
-                        '<b>commit</b></a> or <a href="discard">'
-                        '<b>discard</b></a> changes.')
-            else:
-                self.compressbtn.setEnabled(True)
-                txt = _('You may continue the compress')
-            self.showMessage.emit(txt)
-        self.th = CheckThread(self)
-        self.th.finished.connect(completed)
-        self.th.start()
+    @property
+    def repo(self):
+        return self._repoagent.rawRepo()
+
+    @pyqtSlot(bool)
+    def _checkCompleted(self, clean):
+        if not clean:
+            self.compressbtn.setEnabled(False)
+            txt = _('Before compress, you must <a href="commit">'
+                    '<b>commit</b></a> or <a href="discard">'
+                    '<b>discard</b></a> changes.')
+        else:
+            self.compressbtn.setEnabled(True)
+            txt = _('You may continue the compress')
+        self.showMessage.emit(txt)
 
     def compress(self):
         self.cancelbtn.setShown(False)
         uc = ['update', '--repository', self.repo.root, '--clean', '--rev',
               str(self.revs[1])]
-        rc = ['revert', '--repository', self.repo.root, '--all', '--rev', 
+        rc = ['revert', '--repository', self.repo.root, '--all', '--rev',
               str(self.revs[0])]
         self.repo.incrementBusyCount()
         self.cmd.run(uc, rc)
@@ -125,13 +110,13 @@ class CompressDialog(QDialog):
 
     def commit(self):
         tip, base = self.revs
-        func = hglib.revsetmatch(self.repo.ui, '%s::%s' % (base, tip))
+        func = revset.match(self.repo.ui, '%s::%s' % (base, tip))
         revcount = len(self.repo)
         revs = [c for c in func(self.repo, range(revcount)) if c != base]
         descs = [self.repo[c].description() for c in revs]
         self.repo.opener('cur-message.txt', 'w').write('\n* * *\n'.join(descs))
 
-        dlg = commit.CommitDialog(self.repo, [], {}, self)
+        dlg = commit.CommitDialog(self._repoagent, [], {}, self)
         dlg.finished.connect(dlg.deleteLater)
         dlg.exec_()
         self.showMessage.emit(_('Compress is complete, old history untouched'))
@@ -139,26 +124,18 @@ class CompressDialog(QDialog):
         self.compressbtn.clicked.disconnect(self.commit)
         self.compressbtn.clicked.connect(self.accept)
 
-    def linkActivated(self, cmd):
-        if cmd == 'commit':
-            dlg = commit.CommitDialog(self.repo, [], {}, self)
-            dlg.finished.connect(dlg.deleteLater)
-            dlg.exec_()
-            self.checkStatus()
-        elif cmd == 'discard':
-            labels = [(QMessageBox.Yes, _('&Discard')),
-                      (QMessageBox.No, _('Cancel'))]
-            if not qtlib.QuestionMsgBox(_('Confirm Discard'),
-                     _('Discard outstanding changes to working directory?'),
-                     labels=labels, parent=self):
-                return
-            def finished(ret):
-                self.repo.decrementBusyCount()
-                if ret == 0:
-                    self.checkStatus()
-            cmdline = ['update', '--clean', '--repository', self.repo.root,
-                       '--rev', '.']
-            self.runner = cmdui.Runner(False, self)
-            self.runner.commandFinished.connect(finished)
-            self.repo.incrementBusyCount()
-            self.runner.run(cmdline)
+    def storeSettings(self):
+        s = QSettings()
+        s.setValue('compress/geometry', self.saveGeometry())
+
+    def restoreSettings(self):
+        s = QSettings()
+        self.restoreGeometry(s.value('compress/geometry').toByteArray())
+
+    def accept(self):
+        self.storeSettings()
+        super(CompressDialog, self).accept()
+
+    def reject(self):
+        self.storeSettings()
+        super(CompressDialog, self).reject()
