@@ -11,6 +11,7 @@ import re
 from mercurial import ui, hg, error, commands, match, util, subrepo
 
 from tortoisehg.hgqt import htmlui, visdiff, qtlib, htmldelegate, thgrepo, cmdui, settings
+from tortoisehg.hgqt import filedialogs, fileview
 from tortoisehg.util import paths, hglib, thread2
 from tortoisehg.hgqt.i18n import _
 
@@ -26,11 +27,10 @@ class SearchWidget(QWidget, qtlib.TaskWidget):
     progress = pyqtSignal(QString, object, QString, QString, object)
     revisionSelected = pyqtSignal(int)
 
-    def __init__(self, upats, repo, parent=None, **opts):
+    def __init__(self, repoagent, upats, parent=None, **opts):
         QWidget.__init__(self, parent)
 
         self.thread = None
-        self.setWindowIcon(qtlib.geticon('view-filter'))
 
         mainvbox = QVBoxLayout()
         mainvbox.setSpacing(6)
@@ -107,14 +107,14 @@ class SearchWidget(QWidget, qtlib.TaskWidget):
         revision.toggled.connect(revisiontoggled)
         history.toggled.connect(singlematch.setDisabled)
         revle.setEnabled(False)
-        revle.returnPressed.connect(self.searchActivated)
-        excle.returnPressed.connect(self.searchActivated)
-        incle.returnPressed.connect(self.searchActivated)
-        bt.clicked.connect(self.searchActivated)
+        revle.returnPressed.connect(self.runSearch)
+        excle.returnPressed.connect(self.runSearch)
+        incle.returnPressed.connect(self.runSearch)
+        bt.clicked.connect(self.runSearch)
 
         def updateRecurse(checked):
             try:
-                wctx = repo[None]
+                wctx = self.repo[None]
                 if '.hgsubstate' in wctx:
                     recurse.setEnabled(checked)
                 else:
@@ -141,14 +141,15 @@ class SearchWidget(QWidget, qtlib.TaskWidget):
         frame.setLayout(grid)
         mainvbox.addWidget(frame)
 
-        tv = MatchTree(repo, self)
+        tv = MatchTree(repoagent, self)
         tv.revisionSelected.connect(self.revisionSelected)
         tv.setColumnHidden(COL_REVISION, True)
         tv.setColumnHidden(COL_USER, True)
         mainvbox.addWidget(tv)
-        le.returnPressed.connect(self.searchActivated)
+        le.returnPressed.connect(self.runSearch)
 
-        self.repo = repo
+        self._repoagent = repoagent
+        repo = repoagent.rawRepo()
         self.tv, self.regexple, self.chk, self.recurse = tv, le, chk, recurse
         self.incle, self.excle, self.revle = incle, excle, revle
         self.wctxradio, self.ctxradio, self.aradio = working, revision, history
@@ -156,7 +157,6 @@ class SearchWidget(QWidget, qtlib.TaskWidget):
         self.searchbutton, self.cancelbutton = bt, cbt
         self.regexple.setFocus()
 
-        assert not opts.get('search'), 'cannot start search in constructor'
         if 'rev' in opts or 'all' in opts:
             self.setSearch(upats[0], **opts)
         elif len(upats) >= 1:
@@ -191,6 +191,10 @@ class SearchWidget(QWidget, qtlib.TaskWidget):
             self.showMessage.connect(self.stbar.showMessage)
             self.progress.connect(self.stbar.progress)
 
+    @property
+    def repo(self):
+        return self._repoagent.rawRepo()
+
     def setCompleters(self):
         comp = QCompleter(self.searchhistory, self)
         QShortcut(QKeySequence('CTRL+D'), comp.popup(),
@@ -244,8 +248,6 @@ class SearchWidget(QWidget, qtlib.TaskWidget):
         elif opts.get('rev'):
             self.ctxradio.setChecked(True)
             self.revle.setText(opts['rev'])
-        if opts.get('search'):
-            self.searchActivated()
 
     def stopClicked(self):
         if self.thread and self.thread.isRunning():
@@ -272,11 +274,9 @@ class SearchWidget(QWidget, qtlib.TaskWidget):
         s.setValue('grep/search-'+repoid, self.searchhistory)
         s.setValue('grep/paths-'+repoid, self.pathshistory)
 
-    def closeEvent(self, event):
-        self.saveSettings(QSettings())
-
-    def searchActivated(self):
-        'User pressed [Return] in QLineEdit'
+    @pyqtSlot()
+    def runSearch(self):
+        """Run search for the current pattern in background thread"""
         if self.thread and self.thread.isRunning():
             return
 
@@ -543,10 +543,10 @@ class MatchTree(QTableView):
     revisionSelected = pyqtSignal(int)
     contextmenu = None
 
-    def __init__(self, repo, parent):
+    def __init__(self, repoagent, parent):
         QTableView.__init__(self, parent)
 
-        self.repo = repo
+        self._repoagent = repoagent
         self.pattern = None
         self.icase = False
         self.embedded = parent.parent() is not None
@@ -563,6 +563,10 @@ class MatchTree(QTableView):
         vh.setDefaultSectionSize(20)
         self.horizontalHeader().setStretchLastSection(True)
 
+        self._filedialogs = qtlib.DialogKeeper(MatchTree._createFileDialog,
+                                               MatchTree._genFileDialogKey,
+                                               self)
+
         self.actions = {}
         self.contextmenu = QMenu(self)
         for key, name, func, shortcut in (
@@ -582,6 +586,10 @@ class MatchTree(QTableView):
         self.setModel(MatchModel(self))
         self.selectionModel().selectionChanged.connect(self.onSelectionChanged)
 
+    @property
+    def repo(self):
+        return self._repoagent.rawRepo()
+
     def dragObject(self):
         snapshots = {}
         for index in self.selectionModel().selectedRows():
@@ -649,9 +657,7 @@ class MatchTree(QTableView):
         self.selectedRows = saved
 
     def onAnnotateFile(self):
-        from tortoisehg.hgqt.manifestdialog import run
-        from tortoisehg.hgqt.run import qtrun
-        repo, ui, pattern, icase = self.repo, self.repo.ui, self.pattern, self.icase
+        repo = self.repo
         seen = set()
         for rev, upath, line in self.selectedRows:
             path = hglib.fromunicode(upath)
@@ -665,20 +671,33 @@ class MatchTree(QTableView):
                 root = paths.find_root(abs)
                 if root and abs.startswith(root):
                     path = abs[len(root)+1:]
+                    # TODO: do not instantiate repo here
                     srepo = thgrepo.repository(None, root)
-                    if rev is None:
-                        rev = srepo['.'].rev()
-                    opts = {'repo': srepo, 'canonpath' : path, 'rev' : rev,
-                            'line': line, 'pattern': pattern, 'ignorecase': icase}
-                    qtrun(run, ui, **opts)
+                    srepoagent = srepo._pyqtobj
+                    self._openAnnotateDialog(srepoagent, rev, path, line)
                 else:
                     continue
             else:
-                if rev is None:
-                    rev = repo['.'].rev()
-                opts = {'repo': repo, 'canonpath' : path, 'rev' : rev,
-                        'line': line, 'pattern': pattern, 'ignorecase': icase}
-                qtrun(run, ui, **opts)
+                self._openAnnotateDialog(self._repoagent, rev, path, line)
+
+    def _openAnnotateDialog(self, repoagent, rev, path, line):
+        if rev is None:
+            repo = repoagent.rawRepo()
+            rev = repo['.'].rev()
+
+        dlg = self._filedialogs.open(repoagent, path)
+        dlg.setFileViewMode(fileview.AnnMode)
+        dlg.goto(rev)
+        dlg.showLine(line)
+        dlg.setSearchPattern(hglib.tounicode(self.pattern))
+        dlg.setSearchCaseInsensitive(self.icase)
+
+    def _createFileDialog(self, repoagent, path):
+        return filedialogs.FileLogDialog(repoagent, path)
+
+    def _genFileDialogKey(self, repoagent, path):
+        repo = repoagent.rawRepo()
+        return repo.wjoin(path)
 
     def onViewChangeset(self):
         for rev, path, line in self.selectedRows:
@@ -778,11 +797,27 @@ class MatchModel(QAbstractTableModel):
         assert index.isValid()
         return self.rows[index.row()]
 
-def run(ui, *pats, **opts):
-    repo = thgrepo.repository(ui, path=paths.find_root())
-    upats = [hglib.tounicode(p) for p in pats]
-    search = opts.pop('search', False)
-    dlg = SearchWidget(upats, repo, **opts)
-    if search:
-        dlg.searchActivated()
-    return dlg
+class SearchDialog(QDialog):
+    def __init__(self, repoagent, upats, parent=None, **opts):
+        super(SearchDialog, self).__init__(parent)
+        self.setWindowFlags(Qt.Window)
+        self.setWindowIcon(qtlib.geticon('view-filter'))
+        self.setLayout(QVBoxLayout(self))
+        self._searchwidget = SearchWidget(repoagent, upats, parent=self, **opts)
+        self.layout().addWidget(self._searchwidget)
+
+    def closeEvent(self, event):
+        if not self._searchwidget.canExit():
+            self._searchwidget.stopClicked()
+            event.ignore()
+            return
+
+        self._searchwidget.saveSettings(QSettings())
+        super(SearchDialog, self).closeEvent(event)
+
+    def setSearch(self, upattern, **opts):
+        self._searchwidget.setSearch(upattern, **opts)
+
+    @pyqtSlot()
+    def runSearch(self):
+        self._searchwidget.runSearch()
