@@ -8,18 +8,18 @@
 # GNU General Public License version 2, incorporated herein by reference.
 
 import os, tempfile, re
-from StringIO import StringIO
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
-from mercurial import error, extensions, util, cmdutil
-from tortoisehg.util import hglib, paths
+from mercurial import error, util
+from tortoisehg.util import hglib
 from tortoisehg.hgqt.i18n import _
-from tortoisehg.hgqt import cmdui, lexers, qtlib, thgrepo
+from tortoisehg.hgqt import cmdui, lexers, qtlib
 from tortoisehg.hgqt.hgemail_ui import Ui_EmailDialog
 
 class EmailDialog(QDialog):
     """Dialog for sending patches via email"""
-    def __init__(self, repo, revs, parent=None, outgoing=False,
+
+    def __init__(self, repoagent, revs, parent=None, outgoing=False,
                  outgoingrevs=None):
         """Create EmailDialog for the given repo and revs
 
@@ -31,7 +31,7 @@ class EmailDialog(QDialog):
         """
         super(EmailDialog, self).__init__(parent)
         self.setWindowFlags(Qt.Window)
-        self._repo = repo
+        self._repoagent = repoagent
         self._outgoing = outgoing
         self._outgoingrevs = outgoingrevs or []
 
@@ -42,10 +42,13 @@ class EmailDialog(QDialog):
         self._initpreviewtab()
         self._initenvelopebox()
         self._qui.bundle_radio.toggled.connect(self._updateforms)
+        self._qui.attach_check.toggled.connect(self._updateattachmodes)
+        self._qui.inline_check.toggled.connect(self._updateattachmodes)
         self._initintrobox()
         self._readhistory()
         self._filldefaults()
         self._updateforms()
+        self._updateattachmodes()
         self._readsettings()
         QShortcut(QKeySequence('CTRL+Return'), self, self.accept)
         QShortcut(QKeySequence('Ctrl+Enter'), self, self.accept)
@@ -68,10 +71,13 @@ class EmailDialog(QDialog):
 
     def _readhistory(self):
         s = QSettings()
-        for k in ('to', 'cc', 'from', 'flag'):
+        for k in ('to', 'cc', 'from', 'flag', 'subject'):
             w = getattr(self._qui, '%s_edit' % k)
             w.addItems(s.value('email/%s_history' % k).toStringList())
             w.setCurrentIndex(-1)  # unselect
+        for k in ('body', 'attach', 'inline', 'diffstat'):
+            w = getattr(self._qui, '%s_check' % k)
+            w.setChecked(s.value('email/%s' % k).toBool())
 
     def _writehistory(self):
         def itercombo(w):
@@ -82,23 +88,25 @@ class EmailDialog(QDialog):
                     yield w.itemText(i)
 
         s = QSettings()
-        for k in ('to', 'cc', 'from', 'flag'):
+        for k in ('to', 'cc', 'from', 'flag', 'subject'):
             w = getattr(self._qui, '%s_edit' % k)
             s.setValue('email/%s_history' % k, list(itercombo(w))[:10])
+        for k in ('body', 'attach', 'inline', 'diffstat'):
+            w = getattr(self._qui, '%s_check' % k)
+            s.setValue('email/%s' % k, w.isChecked())
 
     def _initchangesets(self, revs):
-        def purerevs(revs):
-            return hglib.revrange(self._repo,
-                                  iter(str(e) for e in revs))
-
         self._changesets = _ChangesetsModel(self._repo,
-                                            # TODO: [':'] is inefficient
-                                            revs=purerevs(revs or [':']),
-                                            selectedrevs=purerevs(revs),
+                                            revs=revs or list(self._repo),
+                                            selectedrevs=revs,
                                             parent=self)
         self._changesets.dataChanged.connect(self._updateforms)
         self._qui.changesets_view.setModel(self._changesets)
 
+    @property
+    def _repo(self):
+        return self._repoagent.rawRepo()
+
     @property
     def _ui(self):
         return self._repo.ui
@@ -160,7 +168,7 @@ class EmailDialog(QDialog):
         """Generate opts for patchbomb by form values"""
         def headertext(s):
             # QLineEdit may contain newline character
-            return re.sub(r'\s', ' ', hglib.fromunicode(s))
+            return re.sub(r'\s', ' ', unicode(s))
 
         opts['to'] = [headertext(self._qui.to_edit.currentText())]
         opts['cc'] = [headertext(self._qui.cc_edit.currentText())]
@@ -170,10 +178,10 @@ class EmailDialog(QDialog):
 
         if self._qui.bundle_radio.isChecked():
             assert self._outgoing  # only outgoing bundle is supported
-            opts['rev'] = map(str, self._outgoingrevs)
+            opts['rev'] = self._outgoingrevs
             opts['bundle'] = True
         else:
-            opts['rev'] = map(str, self._revs)
+            opts['rev'] = self._revs
 
         def diffformat():
             n = self.getdiffformat()
@@ -186,22 +194,19 @@ class EmailDialog(QDialog):
         opts.update(self.getextraopts())
 
         def writetempfile(s):
-            fd, fname = tempfile.mkstemp(prefix='thg_emaildesc_')
+            fd, fname = tempfile.mkstemp(prefix='thg_emaildesc_',
+                                         dir=qtlib.gettempdir())
             try:
                 os.write(fd, s)
-                return fname
+                return hglib.tounicode(fname)
             finally:
                 os.close(fd)
 
         opts['intro'] = self._qui.writeintro_check.isChecked()
         if opts['intro']:
-            opts['subject'] = headertext(self._qui.subject_edit.text())
-            opts['desc'] = writetempfile(hglib.fromunicode(self._qui.body_edit.toPlainText()))
-            # TODO: change patchbomb not to use temporary file
-
-        # Include the repo in the command so it can be found when thg is not
-        # run from within a hg path
-        opts['repository'] = self._repo.root
+            opts['subject'] = headertext(self._qui.subject_edit.currentText())
+            opts['desc'] = writetempfile(
+                hglib.fromunicode(self._qui.body_edit.toPlainText()))
 
         return opts
 
@@ -211,7 +216,8 @@ class EmailDialog(QDialog):
             if not getattr(self._qui, e).currentText():
                 return False
 
-        if self._qui.writeintro_check.isChecked() and not self._qui.subject_edit.text():
+        if (self._qui.writeintro_check.isChecked()
+            and not self._qui.subject_edit.currentText()):
             return False
 
         if not self._revs:
@@ -238,41 +244,42 @@ class EmailDialog(QDialog):
         if self._introrequired():
             self._qui.writeintro_check.setChecked(True)
 
+    #@pyqtSlot()
+    def _updateattachmodes(self):
+        """Update checkboxes to select the embedding style of the patch"""
+        attachmodes = [self._qui.attach_check, self._qui.inline_check]
+        body = self._qui.body_check
+
+        # --attach and --inline are exclusive
+        if self.sender() in attachmodes and self.sender().isChecked():
+            for w in attachmodes:
+                if w is not self.sender():
+                    w.setChecked(False)
+
+        # --body is mandatory if no attach modes are specified
+        body.setEnabled(util.any(w.isChecked() for w in attachmodes))
+        if not body.isEnabled():
+            body.setChecked(True)
+
     def _initenvelopebox(self):
         for e in ('to_edit', 'from_edit'):
             getattr(self._qui, e).editTextChanged.connect(self._updateforms)
 
     def accept(self):
-        # TODO: want to pass patchbombopts directly
-        def cmdargs(opts):
-            args = []
-            for k, v in opts.iteritems():
-                if isinstance(v, bool):
-                    if v:
-                        args.append('--%s' % k.replace('_', '-'))
-                else:
-                    for e in isinstance(v, basestring) and [v] or v:
-                        args += ['--%s' % k.replace('_', '-'), e]
-
-            return args
-
         hglib.loadextension(self._ui, 'patchbomb')
 
         opts = self._patchbombopts()
-        try:
-            cmd = cmdui.Dialog(['email'] + cmdargs(opts), parent=self)
-            cmd.setWindowTitle(_('Sending Email'))
-            cmd.setShowOutput(False)
-            cmd.finished.connect(cmd.deleteLater)
-            if cmd.exec_():
-                self._writehistory()
-        finally:
-            if 'desc' in opts:
-                os.unlink(opts['desc'])  # TODO: don't use tempfile
+        cmdline = hglib.buildcmdargs('email', **opts)
+        cmd = cmdui.CmdSessionDialog(self)
+        cmd.setWindowTitle(_('Sending Email'))
+        cmd.setLogVisible(False)
+        cmd.setSession(self._repoagent.runCommand(cmdline, self))
+        if cmd.exec_():
+            self._writehistory()
 
     def _initintrobox(self):
         self._qui.intro_box.hide()  # hidden by default
-        self._qui.subject_edit.textChanged.connect(self._updateforms)
+        self._qui.subject_edit.editTextChanged.connect(self._updateforms)
         self._qui.writeintro_check.toggled.connect(self._updateforms)
 
     def _introrequired(self):
@@ -284,13 +291,13 @@ class EmailDialog(QDialog):
             w.setUtf8(True)
             w.setReadOnly(True)
             w.setMarginWidth(1, 0)  # hide area for line numbers
-            self.lexer = lex = lexers.get_diff_lexer(self)
+            self.lexer = lex = lexers.difflexer(self)
             fh = qtlib.getfont('fontdiff')
             fh.changed.connect(self.forwardFont)
             lex.setFont(fh.font())
             w.setLexer(lex)
             # TODO: better way to setup diff lexer
-          
+
         initqsci(self._qui.preview_edit)
 
         self._qui.main_tabs.currentChanged.connect(self._refreshpreviewtab)
@@ -306,40 +313,20 @@ class EmailDialog(QDialog):
         if self._previewtabindex() != index:
             return
 
-        self._qui.preview_edit.setText(self._preview())
-
-    def _preview(self):
-        """Generate preview text by running patchbomb"""
-        def loadpatchbomb():
-            hglib.loadextension(self._ui, 'patchbomb')
-            return extensions.find('patchbomb')
-
-        def wrapui(ui):
-            buf = StringIO()
-            # TODO: common way to prepare pure ui
-            newui = ui.copy()
-            newui.setconfig('ui', 'interactive', False)
-            newui.setconfig('diff', 'git', False)
-            newui.write = lambda *args, **opts: buf.write(''.join(args))
-            newui.status = lambda *args, **opts: None
-            return newui, buf
-
-        def stripheadmsg(s):
-            # TODO: skip until first Content-type: line ??
-            return '\n'.join(s.splitlines()[3:])
-
-        ui, buf = wrapui(self._ui)
+        self._qui.preview_edit.clear()
         opts = self._patchbombopts(test=True)
-        try:
-            # TODO: fix hgext.patchbomb's implementation instead
-            if 'PAGER' in os.environ:
-                del os.environ['PAGER']
+        # TODO: fix hgext.patchbomb's implementation instead
+        if 'PAGER' in os.environ:
+            del os.environ['PAGER']
+        cmdline = hglib.buildcmdargs('email', **opts)
+        sess = self._repoagent.runCommand(cmdline)
+        sess.outputReceived.connect(self._capturepreview)
 
-            loadpatchbomb().patchbomb(ui, self._repo, **opts)
-            return stripheadmsg(hglib.tounicode(buf.getvalue()))
-        finally:
-            if 'desc' in opts:
-                os.unlink(opts['desc'])  # TODO: don't use tempfile
+    @pyqtSlot(unicode, unicode)
+    def _capturepreview(self, msg, label):
+        if label:
+            return  # ignore ui or control message
+        self._qui.preview_edit.append(msg)
 
     def _previewtabindex(self):
         """Index of preview tab"""
@@ -362,7 +349,9 @@ class EmailDialog(QDialog):
     def on_selectnone_button_clicked(self):
         self._changesets.selectNone()
 
-class _ChangesetsModel(QAbstractTableModel):  # TODO: use component of log viewer?
+
+# TODO: use component of log viewer?
+class _ChangesetsModel(QAbstractTableModel):
     _COLUMNS = [('rev', lambda ctx: '%d:%s' % (ctx.rev(), ctx)),
                 ('author', lambda ctx: hglib.username(ctx.user())),
                 ('date', lambda ctx: util.shortdate(ctx.date())),
@@ -457,20 +446,3 @@ class _ChangesetsModel(QAbstractTableModel):  # TODO: use component of log viewe
         first = self.createIndex(0, 0)
         last = self.createIndex(len(self._revs) - 1, 0)
         self.dataChanged.emit(first, last)
-
-def run(ui, *revs, **opts):
-    # TODO: same options as patchbomb
-    if opts.get('rev'):
-        if revs:
-            raise util.Abort(_('use only one form to specify the revision'))
-        revs = opts.get('rev')
-
-    # TODO: repo should be a required argument?
-    repo = opts.get('repo') or thgrepo.repository(ui, paths.find_root())
-
-    try:
-        return EmailDialog(repo, revs, outgoing=opts.get('outgoing', False),
-                           outgoingrevs=opts.get('outgoingrevs', None))
-    except error.RepoLookupError, e:
-        qtlib.ErrorMsgBox(_('Failed to open Email dialog'),
-                          hglib.tounicode(e.message))
