@@ -6,12 +6,12 @@
 # This software may be used and distributed according to the terms of the
 # GNU General Public License version 2, incorporated herein by reference.
 
-from mercurial import hg, error
+from mercurial import error, util
 
 from tortoisehg.util import hglib
 from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt import qtlib, csinfo, i18n, cmdui, status, resolve
-from tortoisehg.hgqt import qscilib, thgrepo, messageentry
+from tortoisehg.hgqt import qscilib, thgrepo, messageentry, commit, wctxcleaner
 
 from PyQt4.QtCore import *
 from PyQt4.QtGui import *
@@ -20,11 +20,12 @@ MARGINS = (8, 0, 0, 0)
 
 class MergeDialog(QWizard):
 
-    def __init__(self, otherrev, repo, parent):
+    def __init__(self, repoagent, otherrev, parent=None):
         super(MergeDialog, self).__init__(parent)
         f = self.windowFlags()
         self.setWindowFlags(f & ~Qt.WindowContextHelpButtonHint)
 
+        repo = repoagent.rawRepo()
         self.otherrev = str(otherrev)
         self.localrev = str(repo['.'].rev())
 
@@ -35,23 +36,31 @@ class MergeDialog(QWizard):
         self.setOption(QWizard.IndependentPages, True)
 
         # set pages
-        self.addPage(SummaryPage(repo, self))
-        self.addPage(MergePage(repo, self))
-        self.addPage(CommitPage(repo, self))
-        self.addPage(ResultPage(repo, self))
+        summarypage = SummaryPage(repoagent, self)
+        self.addPage(summarypage)
+        self.addPage(MergePage(repoagent, self))
+        self.addPage(CommitPage(repoagent, self))
+        self.addPage(ResultPage(repoagent, self))
         self.currentIdChanged.connect(self.pageChanged)
 
+        # move focus to "Next" button so that "Cancel" doesn't eat Enter key
+        summarypage.refreshFinished.connect(
+            self.button(QWizard.NextButton).setFocus)
+
         self.resize(QSize(700, 489).expandedTo(self.minimumSizeHint()))
 
-        repo.repositoryChanged.connect(self.repositoryChanged)
-        repo.configChanged.connect(self.configChanged)
-        self.repo = repo
+        repoagent.repositoryChanged.connect(self.repositoryChanged)
+        repoagent.configChanged.connect(self.configChanged)
 
+    @pyqtSlot()
     def repositoryChanged(self):
-        self.currentPage().repositoryChanged()
+        if self.currentPage():
+            self.currentPage().repositoryChanged()
 
+    @pyqtSlot()
     def configChanged(self):
-        self.currentPage().configChanged()
+        if self.currentPage():
+            self.currentPage().configChanged()
 
     def pageChanged(self, id):
         if id != -1:
@@ -61,16 +70,15 @@ class MergeDialog(QWizard):
         if self.currentPage().canExit():
             super(MergeDialog, self).reject()
 
-    def done(self, ret):
-        self.repo.repositoryChanged.disconnect(self.repositoryChanged)
-        self.repo.configChanged.disconnect(self.configChanged)
-        super(MergeDialog, self).done(ret)
-
 
 class BasePage(QWizardPage):
-    def __init__(self, repo, parent):
+    def __init__(self, repoagent, parent):
         super(BasePage, self).__init__(parent)
-        self.repo = repo
+        self._repoagent = repoagent
+
+    @property
+    def repo(self):
+        return self._repoagent.rawRepo()
 
     def validatePage(self):
         'user pressed NEXT button, can we proceed?'
@@ -89,13 +97,15 @@ class BasePage(QWizardPage):
         pass
 
     def currentPage(self):
-        pass
+        self.wizard().setOption(QWizard.NoDefaultButton, False)
 
     def canExit(self):
         if len(self.repo.parents()) == 2:
             main = _('Do you want to exit?')
-            text = _('To finish merging, you need to commit '
-                     'the working directory.')
+            text = _('To finish merging, you must commit '
+                     'the working directory.\n\n'
+                     'To cancel the merge you can update to one '
+                     'of the merge parent revisions.')
             labels = ((QMessageBox.Yes, _('&Exit')),
                       (QMessageBox.No, _('Cancel')))
             if not qtlib.QuestionMsgBox(_('Confirm Exit'), main, text,
@@ -104,11 +114,13 @@ class BasePage(QWizardPage):
         return True
 
 class SummaryPage(BasePage):
+    refreshFinished = pyqtSignal()
 
-    def __init__(self, repo, parent):
-        super(SummaryPage, self).__init__(repo, parent)
-        self.clean = None
-        self.th = None
+    def __init__(self, repoagent, parent):
+        super(SummaryPage, self).__init__(repoagent, parent)
+        self._wctxcleaner = wctxcleaner.WctxCleaner(repoagent, self)
+        self._wctxcleaner.checkStarted.connect(self._onCheckStarted)
+        self._wctxcleaner.checkFinished.connect(self._onCheckFinished)
 
     ### Override Methods ###
 
@@ -136,12 +148,12 @@ class SummaryPage(BasePage):
         self.layout().addWidget(other_sep)
         try:
             otherCsInfo = create(self.wizard().otherrev)
+            self.layout().addWidget(otherCsInfo)
+            self.otherCsInfo = otherCsInfo
         except error.RepoLookupError:
             qtlib.InfoMsgBox(_('Unable to merge'),
                              _('Merge revision not specified or not found'))
             QTimer.singleShot(0, self.wizard().close)
-        self.layout().addWidget(otherCsInfo)
-        self.otherCsInfo = otherCsInfo
 
         ## current revision
         local_sep = qtlib.LabeledSeparator(_('Merge to (working directory)'))
@@ -181,7 +193,7 @@ class SummaryPage(BasePage):
                  'or <a href="discard"><b>discard</b></a> changes.')
         wd_text = QLabel(text)
         wd_text.setWordWrap(True)
-        wd_text.linkActivated.connect(self.onLinkActivated)
+        wd_text.linkActivated.connect(self._wctxcleaner.runCleaner)
         self.wd_text = wd_text
         self.groups.add(wd_text, 'dirty')
         self.layout().addWidget(wd_text)
@@ -226,7 +238,7 @@ class SummaryPage(BasePage):
 
     def isComplete(self):
         'should Next button be sensitive?'
-        return self.clean or self.field('force').toBool()
+        return self._wctxcleaner.isClean() or self.field('force').toBool()
 
     def validatePage(self):
         'validate that we can continue with the merge'
@@ -256,30 +268,27 @@ class SummaryPage(BasePage):
 
     def canExit(self):
         'can merge tool be closed?'
-        if self.th is not None and self.th.isRunning():
-            self.th.cancel()
-            self.th.wait()
+        if self._wctxcleaner.isChecking():
+            self._wctxcleaner.cancelCheck()
         return True
 
     def currentPage(self):
+        super(SummaryPage, self).currentPage()
         self.refresh()
 
     def refresh(self):
-        if self.th is None:
-            self.th = CheckThread(self.repo, self)
-            self.th.finished.connect(self.threadFinished)
-        if self.th.isRunning():
-            return
-        self.groups.set_visible(True, 'prog')
-        self.th.start()
+        self._wctxcleaner.check()
 
-    def threadFinished(self):
+    @pyqtSlot()
+    def _onCheckStarted(self):
+        self.groups.set_visible(True, 'prog')
+
+    @pyqtSlot(bool, int)
+    def _onCheckFinished(self, clean, parents):
         self.groups.set_visible(False, 'prog')
-        if self.th.canceled:
+        if self._wctxcleaner.isCheckCanceled():
             return
-        dirty, parents = self.th.results
-        self.clean = not dirty
-        if dirty:
+        if not clean:
             self.groups.set_visible(parents == 2, 'merged')
             self.groups.set_visible(parents == 1, 'dirty')
             self.wd_status.set_status(_('<b>Uncommitted local changes '
@@ -289,53 +298,19 @@ class SummaryPage(BasePage):
             self.groups.set_visible(False, 'merged')
             self.wd_status.set_status(_('Clean', 'working dir state'), True)
         self.completeChanged.emit()
+        self.refreshFinished.emit()
 
     @pyqtSlot(QString)
     def onLinkActivated(self, cmd):
-        cmd = hglib.fromunicode(cmd)
-        repo = self.repo
-        if cmd == 'commit':
-            from tortoisehg.hgqt import commit
-            dlg = commit.CommitDialog(repo, [], {}, self)
-            dlg.finished.connect(dlg.deleteLater)
-            dlg.exec_()
-            self.refresh()
-        elif cmd == 'shelve':
-            from tortoisehg.hgqt import shelve
-            dlg = shelve.ShelveDialog(repo, self.wizard())
-            dlg.finished.connect(dlg.deleteLater)
-            dlg.exec_()
-            self.refresh()
-        elif cmd.startswith('discard'):
-            if cmd != 'discard:noconfirm':
-                labels = [(QMessageBox.Yes, _('&Discard')),
-                          (QMessageBox.No, _('Cancel'))]
-                if not qtlib.QuestionMsgBox(_('Confirm Discard'),
-                         _('Discard outstanding changes to working directory?'),
-                         labels=labels, parent=self):
-                    return
-            def finished(ret):
-                repo.decrementBusyCount()
-                self.refresh()
-            cmdline = ['update', '--clean', '--repository', repo.root,
-                       '--rev', '.']
-            self.runner = cmdui.Runner(True, self)
-            self.runner.commandFinished.connect(finished)
-            repo.incrementBusyCount()
-            self.runner.run(cmdline)
-        elif cmd == 'view':
-            dlg = status.StatusDialog(repo, [], {}, self)
-            dlg.exec_()
-            self.refresh()
-        elif cmd == 'skip':
+        if cmd == 'skip':
             self.wizard().next()
         else:
-            raise 'unknown command: %s' % cmd
+            self._wctxcleaner.runCleaner(cmd)
 
 
 class MergePage(BasePage):
-    def __init__(self, repo, parent):
-        super(MergePage, self).__init__(repo, parent)
+    def __init__(self, repoagent, parent):
+        super(MergePage, self).__init__(repoagent, parent)
         self.mergecomplete = False
 
         self.setTitle(_('Merging...'))
@@ -360,6 +335,7 @@ class MergePage(BasePage):
         self.layout().addWidget(self.autonext)
 
     def currentPage(self):
+        super(MergePage, self).currentPage()
         if self.field('discard').toBool():
             # '.' is safer than self.localrev, in case the user has
             # pulled a fast one on us and updated from the CLI
@@ -423,7 +399,7 @@ class MergePage(BasePage):
     @pyqtSlot(QString)
     def onLinkActivated(self, cmd):
         if cmd == 'resolve':
-            dlg = resolve.ResolveDialog(self.repo, self)
+            dlg = resolve.ResolveDialog(self._repoagent, self)
             dlg.finished.connect(dlg.deleteLater)
             dlg.exec_()
             if self.autonext.isChecked():
@@ -433,14 +409,16 @@ class MergePage(BasePage):
 
 class CommitPage(BasePage):
 
-    def __init__(self, repo, parent):
-        super(CommitPage, self).__init__(repo, parent)
+    def __init__(self, repoagent, parent):
+        super(CommitPage, self).__init__(repoagent, parent)
 
         self.setTitle(_('Commit merge results'))
         self.setSubTitle(' ')
         self.setLayout(QVBoxLayout())
         self.setCommitPage(True)
 
+        repo = repoagent.rawRepo()
+
         # csinfo
         def label_func(widget, item, ctx):
             if item == 'rev':
@@ -493,6 +471,7 @@ class CommitPage(BasePage):
                                     withupdate=True)
         mergeCsInfo.linkActivated.connect(self.onLinkActivated)
         self.layout().addWidget(mergeCsInfo)
+        self.mergeCsInfo = mergeCsInfo
 
         # commit message area
         msg_sep = qtlib.LabeledSeparator(_('Commit message'))
@@ -511,6 +490,8 @@ class CommitPage(BasePage):
         self.cmd.setShowOutput(False)
         self.layout().addWidget(self.cmd)
 
+        self.delayednext = False
+
         def tryperform():
             if self.isComplete():
                 self.wizard().next()
@@ -525,8 +506,29 @@ class CommitPage(BasePage):
         self.skiplast.setChecked(checked)
         self.layout().addWidget(self.skiplast)
 
+        hblayout = QHBoxLayout()
+        self.opts = commit.readopts(self.repo.ui)
+        self.optionsbtn = QPushButton(_('Commit Options'))
+        self.optionsbtn.clicked.connect(self.details)
+        hblayout.addWidget(self.optionsbtn)
+        self.optionslabelfmt = _('<b>Selected Options:</b> %s')
+        self.optionslabel = QLabel('')
+        hblayout.addWidget(self.optionslabel)
+        hblayout.addStretch()
+        self.layout().addLayout(hblayout)
+
+        self.setButtonText(QWizard.CommitButton, _('Commit Now'))
+        # The cancel button does not really "cancel" the merge
+        self.setButtonText(QWizard.CancelButton, _('Commit Later'))
+
+        # Update the options label
+        self.refresh()
+
     def refresh(self):
-        pass
+        opts = commit.commitopts2str(self.opts)
+        self.optionslabel.setText(self.optionslabelfmt
+            % hglib.tounicode(opts))
+        self.optionslabel.setVisible(bool(opts))
 
     def cleanupPage(self):
         s = QSettings()
@@ -534,6 +536,10 @@ class CommitPage(BasePage):
         self.msgEntry.saveSettings(s, 'merge/message')
 
     def currentPage(self):
+        super(CommitPage, self).currentPage()
+        self.wizard().setOption(QWizard.NoDefaultButton, True)
+        self.mergeCsInfo.update()  # show post-merge state
+
         engmsg = self.repo.ui.configbool('tortoisehg', 'engmsg', False)
         wctx = self.repo[None]
         if wctx.p1().branch() == wctx.p2().branch():
@@ -550,7 +556,7 @@ class CommitPage(BasePage):
     @pyqtSlot(QString)
     def onLinkActivated(self, cmd):
         if cmd == 'view':
-            dlg = status.StatusDialog(self.repo, [], {}, self)
+            dlg = status.StatusDialog(self._repoagent, [], {}, self)
             dlg.exec_()
             self.refresh()
 
@@ -558,15 +564,17 @@ class CommitPage(BasePage):
         return len(self.repo.parents()) == 2 and len(self.msgEntry.text()) > 0
 
     def validatePage(self):
+
+        if self.cmd.core.running():
+            return False
+
         if len(self.repo.parents()) == 1:
             # commit succeeded, repositoryChanged() called wizard().next()
             if self.skiplast.isChecked():
                 self.wizard().close()
             return True
-        if self.cmd.core.running():
-            return False
 
-        user = qtlib.getCurrentUsername(self, self.repo)
+        user = qtlib.getCurrentUsername(self, self.repo, self.opts)
         if not user:
             return False
 
@@ -576,6 +584,28 @@ class CommitPage(BasePage):
         message = hglib.fromunicode(self.msgEntry.text())
         cmdline = ['commit', '--verbose', '--message', message,
                    '--repository', self.repo.root, '--user', user]
+        if self.opts.get('recurseinsubrepos'):
+            cmdline.append('--subrepos')
+        try:
+            date = self.opts.get('date')
+            if date:
+                util.parsedate(date)
+                dcmd = ['--date', date]
+            else:
+                dcmd = []
+        except error.Abort, e:
+            if e.hint:
+                err = _('%s (hint: %s)') % (hglib.tounicode(str(e)),
+                                            hglib.tounicode(e.hint))
+            else:
+                err = hglib.tounicode(str(e))
+            qtlib.WarningMsgBox(_('TortoiseHg Merge Commit'),
+                _('Error creating interpreting commit date (%s).\n'
+                  'Using current date instead.'), err)
+            dcmd = []
+
+        cmdline += dcmd
+
         commandlines = [cmdline]
         pushafter = self.repo.ui.config('tortoisehg', 'cipushafter')
         if pushafter:
@@ -589,16 +619,39 @@ class CommitPage(BasePage):
     def repositoryChanged(self):
         'repository has detected a change to changelog or parents'
         if len(self.repo.parents()) == 1:
-            self.wizard().next()
+            if self.cmd.core.running():
+                # call self.wizard().next() after the current command finishes
+                self.delayednext = True
+            else:
+                self.wizard().next()
 
     def onCommandFinished(self, ret):
         self.repo.decrementBusyCount()
+        if self.delayednext:
+            self.delayednext = False
+            self.wizard().next()
         self.completeChanged.emit()
 
+    def readUserHistory(self):
+        'Load user history from the global commit settings'
+        s = QSettings()
+        userhist = s.value('commit/userhist').toStringList()
+        userhist = [u for u in userhist if u]
+        return userhist
+
+    def details(self):
+        self.userhist = self.readUserHistory()
+        dlg = commit.DetailsDialog(self.opts, self.userhist, self, mode='merge')
+        dlg.finished.connect(dlg.deleteLater)
+        dlg.setWindowFlags(Qt.Sheet)
+        dlg.setWindowModality(Qt.WindowModal)
+        if dlg.exec_() == QDialog.Accepted:
+            self.opts.update(dlg.outopts)
+            self.refresh()
 
 class ResultPage(BasePage):
-    def __init__(self, repo, parent):
-        super(ResultPage, self).__init__(repo, parent)
+    def __init__(self, repoagent, parent):
+        super(ResultPage, self).__init__(repoagent, parent)
         self.setTitle(_('Finished'))
         self.setSubTitle(' ')
         self.setFinalPage(True)
@@ -612,45 +665,6 @@ class ResultPage(BasePage):
         self.layout().addStretch(1)
 
     def currentPage(self):
+        super(ResultPage, self).currentPage()
         self.mergeCsInfo.update(self.repo['tip'])
         self.wizard().setOption(QWizard.NoCancelButton, True)
-
-
-class CheckThread(QThread):
-    def __init__(self, repo, parent):
-        QThread.__init__(self, parent)
-        self.repo = hg.repository(repo.ui, repo.root)
-        self.results = (False, 1)
-        self.canceled = False
-
-    def run(self):
-        self.repo.dirstate.invalidate()
-        unresolved = False
-        for root, path, status in thgrepo.recursiveMergeStatus(self.repo):
-            if self.canceled:
-                return
-            if status == 'u':
-                unresolved = True
-                break
-        wctx = self.repo[None]
-        try:
-            dirty = bool(wctx.dirty()) or unresolved
-            self.results = (dirty, len(wctx.parents()))
-        except EnvironmentError:
-            self.results = (True, len(wctx.parents()))
-
-    def cancel(self):
-        self.canceled = True
-
-def run(ui, *pats, **opts):
-    from tortoisehg.util import paths
-    rev = opts.get('rev') or None
-    if not rev and len(pats):
-        rev = pats[0]
-    if not rev:
-        import sys
-        qtlib.InfoMsgBox(_('Unable to merge'),
-                         _('Merge revision not specified or not found'))
-        sys.exit()
-    repo = thgrepo.repository(ui, path=paths.find_root())
-    return MergeDialog(rev, repo, None)
