@@ -6,13 +6,11 @@
 # of the GNU General Public License, incorporated herein by reference.
 
 import cStringIO
-import os
 
-from mercurial import hg, util, patch, commands
+from mercurial import patch
 from hgext import record
 
 from tortoisehg.util import hglib
-from tortoisehg.util.patchctx import patchctx
 from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt import qtlib, qscilib, lexers
 
@@ -23,7 +21,7 @@ from PyQt4 import Qsci
 qsci = Qsci.QsciScintilla
 
 class RejectsDialog(QDialog):
-    def __init__(self, path, parent):
+    def __init__(self, ui, path, parent=None):
         super(RejectsDialog, self).__init__(parent)
         self.setWindowTitle(_('Merge rejected patch chunks into %s') %
                             hglib.tounicode(path))
@@ -35,8 +33,6 @@ class RejectsDialog(QDialog):
         editor.setBraceMatching(qsci.SloppyBraceMatch)
         editor.setFolding(qsci.BoxedTreeFoldStyle)
         editor.installEventFilter(qscilib.KeyPressInterceptor(self))
-        editor.setContextMenuPolicy(Qt.CustomContextMenu)
-        editor.customContextMenuRequested.connect(self.menuRequested)
         self.baseLineColor = editor.markerDefine(qsci.Background, -1)
         editor.setMarkerBackgroundColor(QColor('lightblue'), self.baseLineColor)
         self.layout().addWidget(editor, 3)
@@ -48,7 +44,7 @@ class RejectsDialog(QDialog):
         def showsearchbar():
             searchbar.show()
             searchbar.setFocus(Qt.OtherFocusReason)
-        QShortcut(QKeySequence.Find, self, showsearchbar)
+        qtlib.newshortcutsforstdkey(QKeySequence.Find, self, showsearchbar)
         self.layout().addWidget(searchbar)
 
         hbox = QHBoxLayout()
@@ -91,14 +87,14 @@ class RejectsDialog(QDialog):
         self.editor.loadSettings(s, 'rejects/editor')
         self.rejectbrowser.loadSettings(s, 'rejects/rejbrowse')
 
-        f = QFile(path)
+        f = QFile(hglib.tounicode(path))
         if not f.open(QIODevice.ReadOnly):
             qtlib.ErrorMsgBox(_('Unable to merge rejects'),
                               _("Can't read this file (maybe deleted)"))
             self.hide()
             QTimer.singleShot(0, self.reject)
             return
-        earlybytes = f.readData(4096)
+        earlybytes = f.read(4096)
         if '\0' in earlybytes:
             qtlib.ErrorMsgBox(_('Unable to merge rejects'),
                               _('This appears to be a binary file'))
@@ -109,7 +105,7 @@ class RejectsDialog(QDialog):
         f.seek(0)
         editor.read(f)
         editor.setModified(False)
-        lexer = lexers.get_lexer(path, earlybytes, self)
+        lexer = lexers.getlexer(ui, path, earlybytes, self)
         editor.setLexer(lexer)
         editor.setMarginLineNumbers(1, True)
         editor.setMarginWidth(1, str(editor.lines())+'X')
@@ -135,10 +131,6 @@ class RejectsDialog(QDialog):
         self.unresolved.setDisabled(True)
         QTimer.singleShot(0, lambda: self.chunklist.setCurrentRow(0))
 
-    def menuRequested(self, point):
-        point = self.editor.viewport().mapToGlobal(point)
-        return self.editor.createStandardContextMenu().exec_(point)
-
     def updateChunkList(self):
         self.updating = True
         self.chunklist.clear()
@@ -177,11 +169,12 @@ class RejectsDialog(QDialog):
         buf = cStringIO.StringIO()
         chunk = self.chunks[row]
         chunk.write(buf)
+        startline = max(chunk.fromline-1, 0)
         self.rejectbrowser.showChunk(buf.getvalue().splitlines(True)[1:])
-        self.editor.setCursorPosition(chunk.fromline-1, 0)
-        self.editor.ensureLineVisible(chunk.fromline-1)
+        self.editor.setCursorPosition(startline, 0)
+        self.editor.ensureLineVisible(startline)
         self.editor.markerDeleteAll(-1)
-        self.editor.markerAdd(chunk.fromline-1, self.baseLineColor)
+        self.editor.markerAdd(startline, self.baseLineColor)
         self.resolved.setEnabled(not chunk.resolved)
         self.unresolved.setEnabled(chunk.resolved)
 
@@ -212,9 +205,12 @@ class RejectsDialog(QDialog):
                 acceptresolution = True
 
         if acceptresolution:
-            f = QFile(self.path)
-            f.open(QIODevice.WriteOnly)
-            self.editor.write(f)
+            f = QFile(hglib.tounicode(self.path))
+            saved = f.open(QIODevice.WriteOnly) and self.editor.write(f)
+            if not saved:
+                qtlib.ErrorMsgBox(_('Unable to save file'),
+                                  f.errorString(), parent=self)
+                return
             self.saveSettings()
             super(RejectsDialog, self).accept()
 
@@ -232,8 +228,6 @@ class RejectBrowser(qscilib.Scintilla):
         self.setUtf8(True)
 
         self.installEventFilter(qscilib.KeyPressInterceptor(self))
-        self.setContextMenuPolicy(Qt.CustomContextMenu)
-        self.customContextMenuRequested.connect(self.menuRequested)
         self.setCaretLineVisible(False)
 
         self.setMarginType(1, qsci.SymbolMargin)
@@ -249,13 +243,9 @@ class RejectBrowser(qscilib.Scintilla):
         mask = (1 << self.addedMark) | (1 << self.removedMark) | \
                (1 << self.addedColor) | (1 << self.removedColor)
         self.setMarginMarkerMask(1, mask)
-        lexer = lexers.get_diff_lexer(self)
+        lexer = lexers.difflexer(self)
         self.setLexer(lexer)
 
-    def menuRequested(self, point):
-        point = self.viewport().mapToGlobal(point)
-        return self.createStandardContextMenu().exec_(point)
-
     def showChunk(self, lines):
         utext = []
         added = []
@@ -274,14 +264,3 @@ class RejectBrowser(qscilib.Scintilla):
         for i in removed:
             self.markerAdd(i, self.removedMark)
             self.markerAdd(i, self.removedColor)
-
-def run(ui, *pats, **opts):
-    if len(pats) != 1:
-        qtlib.ErrorMsgBox(_('Filename required'),
-                          _('You must provide the path to a file'))
-        import sys; sys.exit()
-    path = pats[0]
-    if path.endswith('.rej'):
-        path = path[:-4]
-    dlg = RejectsDialog(path, None)
-    return dlg
