@@ -227,9 +227,10 @@ class RepoRegistryView(QDockWidget):
     removeRepo = pyqtSignal(QString)
     progressReceived = pyqtSignal(QString, object, QString, QString, object)
 
-    def __init__(self, parent):
+    def __init__(self, repomanager, parent):
         QDockWidget.__init__(self, parent)
 
+        self._repomanager = repomanager
         self.watcher = None
         self._setupSettingActions()
 
@@ -262,7 +263,7 @@ class RepoRegistryView(QDockWidget):
         self._updateSettingActions()
 
         sfile = settingsfilename()
-        model = repotreemodel.RepoTreeModel(sfile, self,
+        model = repotreemodel.RepoTreeModel(sfile, repomanager, self,
             showShortPaths=self._isSettingEnabled('showShortPaths'))
         tv.setModel(model)
 
@@ -366,7 +367,8 @@ class RepoRegistryView(QDockWidget):
     def reloadModel(self):
         oldmodel = self.tview.model()
         activeroot = oldmodel.repoRoot(oldmodel.activeRepoIndex())
-        newmodel = repotreemodel.RepoTreeModel(settingsfilename(), self,
+        newmodel = repotreemodel.RepoTreeModel(settingsfilename(),
+            self._repomanager, self,
             self._isSettingEnabled('showShortPaths'))
         self.tview.setModel(newmodel)
         oldmodel.deleteLater()
@@ -442,6 +444,10 @@ class RepoRegistryView(QDockWidget):
              ("addsubrepo", _("A&dd Subrepository..."), 'thg-add-subrepo',
                 _("Convert an existing repository into a subrepository"),
                 self.addSubrepo),
+             ("removesubrepo", _("Remo&ve Subrepository..."),
+                'thg-remove-subrepo',
+                _("Remove this subrepository from the current revision"),
+                self.removeSubrepo),
              ("copypath", _("Copy &Path"), '',
                 _("Copy the root path of the repository to the clipboard"),
                 self.copyPath),
@@ -582,8 +588,6 @@ class RepoRegistryView(QDockWidget):
                         'as: "%s"') % (sroot, root, srelroot), parent=self)
                     return
                 else:
-                    # Already a subrepo!
-
                     # Read the current .hgsub file contents
                     lines = []
                     hasHgsub = os.path.exists(repo.wjoin('.hgsub'))
@@ -602,6 +606,8 @@ class RepoRegistryView(QDockWidget):
                     # Make sure that the selected subrepo (or one of its
                     # subrepos!) is not already on the .hgsub file
                     linesep = ''
+                    # On Windows case is unimportant, while on posix it is
+                    srelrootnormcase = os.path.normcase(srelroot)
                     for line in lines:
                         line = hglib.tounicode(line)
                         spath = line.split("=")[0].strip()
@@ -610,7 +616,7 @@ class RepoRegistryView(QDockWidget):
                         if not linesep:
                             linesep = hglib.getLineSeparator(line)
                         spath = util.pconvert(spath)
-                        if line.startswith(srelroot):
+                        if os.path.normcase(spath) == srelrootnormcase:
                             qtlib.WarningMsgBox(
                                 _('Failed to add repository'),
                                 _('The .hgsub file already contains the '
@@ -647,6 +653,64 @@ class RepoRegistryView(QDockWidget):
                             % root, parent=self)
                 return
 
+    def removeSubrepo(self):
+        'menu action handler for removing an existing subrepository'
+        path = hglib.tounicode(self.selitem.internalPointer().rootpath())
+        containerpath = os.path.normpath(os.path.join(path, '..'))
+        root = paths.find_root(containerpath)
+        relsubpath = os.path.normcase(os.path.normpath(path[1+len(root):]))
+        hgsubfilename = os.path.join(root, '.hgsub')
+
+        try:
+            f = open(hgsubfilename, 'r')
+            hgsub = []
+            found = False
+            for line in f.readlines():
+                spath = os.path.normcase(
+                    os.path.normpath(
+                        line.split('=')[0].strip()))
+                if spath != relsubpath:
+                    hgsub.append(line)
+                else:
+                    found = True
+            f.close()
+        except IOError:
+            qtlib.ErrorMsgBox(_('Could not open .hgsub file'),
+                _('Cannot read the .hgsub file.<p>'
+                  'Subrepository removal failed.'),
+                parent=self)
+            return
+
+        if not found:
+            qtlib.WarningMsgBox(_('Subrepository not found'),
+                _('The selected subrepository was not found '
+                  'on the .hgsub file.<p>'
+                  'Perhaps it has already been removed?'),
+                parent=self)
+            return
+        choices = (_('&Yes'), _('&No'))
+        answer = qtlib.CustomPrompt(_('Remove the selected repository?'),
+            _('Do you really want to remove the repository "<i>%s</i>" '
+              'from its parent repository "<i>%s</i>"') % (relsubpath, root),
+            self, choices=choices, default=choices[0]).run()
+        if answer != 0:
+            return
+        try:
+            f = open(hgsubfilename, 'w')
+            f.writelines(hgsub)
+            f.close()
+            qtlib.InfoMsgBox(_('Subrepository removed from .hgsub'),
+                _('The selected subrepository has been removed '
+                  'from the .hgsub file.<p>'
+                  'Remember that you must commit this .hgsub change in order '
+                  'to complete the removal of the subrepository!'),
+                parent=self)
+        except IOError:
+            qtlib.ErrorMsgBox(_('Could not update .hgsub file'),
+                _('Cannot update the .hgsub file.<p>'
+                  'Subrepository removal failed.'),
+                parent=self)
+
     def startSettings(self):
         root = self.selitem.internalPointer().rootpath()
         sd = settings.SettingsDialog(configrepo=True, focus='web.name',
@@ -714,7 +778,8 @@ class RepoRegistryView(QDockWidget):
 
     def sortbypath(self):
         childs = self.selitem.internalPointer().childs
-        self.tview.model().sortchilds(childs, lambda x: util.normpath(x.rootpath()))
+        self.tview.model().sortchilds(
+            childs, lambda x: os.path.normcase(util.normpath(x.rootpath())))
 
     def sortbyhgsub(self):
         ip = self.selitem.internalPointer()
@@ -736,19 +801,6 @@ class RepoRegistryView(QDockWidget):
                 return 0
         self.tview.model().sortchilds(ip.childs, keyfunc)
 
-    @pyqtSlot(QString, QString)
-    def shortNameChanged(self, uroot, uname):
-        it = self.tview.model().getRepoItem(hglib.fromunicode(uroot))
-        if it:
-            it.setShortName(uname)
-            self.tview.model().layoutChanged.emit()
-
-    @pyqtSlot(QString, object)
-    def baseNodeChanged(self, uroot, basenode):
-        it = self.tview.model().getRepoItem(hglib.fromunicode(uroot))
-        if it:
-            it.setBaseNode(basenode)
-
     def _scanAddedRepo(self, index):
         m = self.tview.model()
         invalidpaths = m.loadSubrepos(index)
