@@ -152,6 +152,7 @@ class RepoItem(RepoTreeItem):
     def isRepo(self):
         return True
 
+    # TODO: return unicode instead of localstr because shortname() is unicode
     def rootpath(self):
         return self._root
 
@@ -185,7 +186,7 @@ class RepoItem(RepoTreeItem):
             if not self._valid:
                 ico = qtlib.getoverlaidicon(ico, qtlib.geticon('dialog-warning'))
             return ico
-        elif role == Qt.DisplayRole:
+        elif role in (Qt.DisplayRole, Qt.EditRole):
             return [self.shortname, self.shortpath][column]()
 
     def getCommonPath(self):
@@ -267,7 +268,6 @@ class RepoItem(RepoTreeItem):
                     sri._valid = False
                     invalidRepoList.append(repo.wjoin(subpath))
                     return invalidRepoList
-                    continue
 
                 if subtype == 'hg':
                     # Only recurse into mercurial subrepos
@@ -368,8 +368,8 @@ class SubrepoItem(RepoItem):
             return super(SubrepoItem, self).data(column, role)
 
     def menulist(self):
-        acts = ['open', 'clone', 'addsubrepo', None, 'explore',
-                'terminal', 'copypath']
+        acts = ['open', 'clone', None, 'addsubrepo', 'removesubrepo',
+                None, 'explore', 'terminal', 'copypath']
         if self.childCount() > 0:
             acts.extend([None, (_('&Sort'), ['sortbyname', 'sortbyhgsub'])])
         acts.extend([None, 'settings'])
