@@ -10,8 +10,6 @@ from PyQt4.QtGui import *
 
 import os
 
-from mercurial import merge as mergemod
-
 from tortoisehg.util import hglib
 from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt import qtlib, cmdui, csinfo, visdiff, thgrepo
@@ -19,12 +17,13 @@ from tortoisehg.hgqt import qtlib, cmdui, csinfo, visdiff, thgrepo
 MARGINS = (8, 0, 0, 0)
 
 class ResolveDialog(QDialog):
-    def __init__(self, repo, parent=None):
+    def __init__(self, repoagent, parent=None):
         super(ResolveDialog, self).__init__(parent)
+        self._repoagent = repoagent
+        repo = repoagent.rawRepo()
         self.setWindowFlags(Qt.Window)
-        self.setWindowTitle(_('Resolve conflicts - %s') % repo.displayname)
+        self.setWindowTitle(_('Resolve Conflicts - %s') % repo.displayname)
         self.setWindowIcon(qtlib.geticon('hg-merge'))
-        self.repo = repo
 
         self.setLayout(QVBoxLayout())
         self.layout().setSpacing(5)
@@ -88,19 +87,19 @@ class ResolveDialog(QDialog):
         vbox = QVBoxLayout()
         vbox.setContentsMargins(*MARGINS)
         hbox.addLayout(vbox)
-        auto = QPushButton(_('Mercurial Resolve'))
+        auto = QPushButton(_('Mercurial Re&solve'))
         auto.setToolTip(_('Attempt automatic (trivial) merge'))
         auto.clicked.connect(lambda: self.merge('internal:merge'))
-        manual = QPushButton(_('Tool Resolve'))
+        manual = QPushButton(_('Tool &Resolve'))
         manual.setToolTip(_('Merge using selected merge tool'))
         manual.clicked.connect(self.merge)
-        local = QPushButton(_('Take Local'))
+        local = QPushButton(_('&Take Local'))
         local.setToolTip(_('Accept the local file version (yours)'))
         local.clicked.connect(lambda: self.merge('internal:local'))
-        other = QPushButton(_('Take Other'))
+        other = QPushButton(_('Take &Other'))
         other.setToolTip(_('Accept the other file version (theirs)'))
         other.clicked.connect(lambda: self.merge('internal:other'))
-        res = QPushButton(_('Mark as Resolved'))
+        res = QPushButton(_('&Mark as Resolved'))
         res.setToolTip(_('Mark this file as resolved'))
         res.clicked.connect(self.markresolved)
         vbox.addWidget(auto)
@@ -111,6 +110,29 @@ class ResolveDialog(QDialog):
         vbox.addStretch(1)
         self.ubuttons = (auto, manual, local, other, res)
 
+        self.utree.setContextMenuPolicy(Qt.CustomContextMenu)
+        self.utreecmenu = QMenu(self)
+        cmauto = self.utreecmenu.addAction(_('Mercurial Re&solve'))
+        cmauto.triggered.connect(lambda: self.merge('internal:merge'))
+        cmmanual = self.utreecmenu.addAction(_('Tool &Resolve'))
+        cmmanual.triggered.connect(self.merge)
+        cmlocal = self.utreecmenu.addAction(_('&Take Local'))
+        cmlocal.triggered.connect(lambda: self.merge('internal:local'))
+        cmother = self.utreecmenu.addAction(_('Take &Other'))
+        cmother.triggered.connect(lambda: self.merge('internal:other'))
+        cmres = self.utreecmenu.addAction(_('&Mark as Resolved'))
+        cmres.triggered.connect(self.markresolved)
+        self.utreecmenu.addSeparator()
+        cmdiffLocToAnc = self.utreecmenu.addAction(_('Diff &Local to Ancestor'))
+        cmdiffLocToAnc.triggered.connect(self.diffLocToAnc)
+        cmdiffOthToAnc = self.utreecmenu.addAction(_('&Diff Other to Ancestor'))
+        cmdiffOthToAnc.triggered.connect(self.diffOthToAnc)
+        self.umenuitems = (cmauto, cmmanual, cmlocal, cmother, cmres,
+                           cmdiffLocToAnc, cmdiffOthToAnc)
+        self.utree.customContextMenuRequested.connect(self.utreeMenuRequested)
+
+        self.utree.doubleClicked.connect(self.utreeDoubleClicked)
+
         res = qtlib.LabeledSeparator(_('Resolved conflicts'))
         self.layout().addWidget(res)
 
@@ -125,19 +147,19 @@ class ResolveDialog(QDialog):
         vbox = QVBoxLayout()
         vbox.setContentsMargins(*MARGINS)
         hbox.addLayout(vbox)
-        edit = QPushButton(_('Edit File'))
+        edit = QPushButton(_('&Edit File'))
         edit.setToolTip(_('Edit resolved file'))
         edit.clicked.connect(self.edit)
-        v3way = QPushButton(_('3-Way Diff'))
+        v3way = QPushButton(_('3-&Way Diff'))
         v3way.setToolTip(_('Visual three-way diff'))
         v3way.clicked.connect(self.v3way)
-        vp0 = QPushButton(_('Diff to Local'))
+        vp0 = QPushButton(_('Diff to &Local'))
         vp0.setToolTip(_('Visual diff between resolved file and first parent'))
         vp0.clicked.connect(self.vp0)
-        vp1 = QPushButton(_('Diff to Other'))
+        vp1 = QPushButton(_('&Diff to Other'))
         vp1.setToolTip(_('Visual diff between resolved file and second parent'))
         vp1.clicked.connect(self.vp1)
-        ures = QPushButton(_('Mark as Unresolved'))
+        ures = QPushButton(_('Mark as &Unresolved'))
         ures.setToolTip(_('Mark this file as unresolved'))
         ures.clicked.connect(self.markunresolved)
         vbox.addWidget(edit)
@@ -149,6 +171,24 @@ class ResolveDialog(QDialog):
         self.rbuttons = (edit, vp0, ures)
         self.rmbuttons = (vp1, v3way)
 
+        self.rtree.setContextMenuPolicy(Qt.CustomContextMenu)
+        self.rtreecmenu = QMenu(self)
+        cmedit = self.rtreecmenu.addAction(_('&Edit File'))
+        cmedit.triggered.connect(self.edit)
+        cmv3way = self.rtreecmenu.addAction(_('3-&Way Diff'))
+        cmv3way.triggered.connect(self.v3way)
+        cmvp0 = self.rtreecmenu.addAction(_('Diff to &Local'))
+        cmvp0.triggered.connect(self.vp0)
+        cmvp1 = self.rtreecmenu.addAction(_('&Diff to Other'))
+        cmvp1.triggered.connect(self.vp1)
+        cmures = self.rtreecmenu.addAction(_('Mark as &Unresolved'))
+        cmures.triggered.connect(self.markunresolved)
+        self.rmenuitems = (cmedit, cmvp0, cmures)
+        self.rmmenuitems = (cmvp1, cmv3way)
+        self.rtree.customContextMenuRequested.connect(self.rtreeMenuRequested)
+
+        self.rtree.doubleClicked.connect(self.v3way)
+
         hbox = QHBoxLayout()
         hbox.setContentsMargins(*MARGINS)
         hbox.setSpacing(4)
@@ -178,14 +218,14 @@ class ResolveDialog(QDialog):
         self.refresh()
         self.utree.selectAll()
         self.utree.setFocus()
-        repo.configChanged.connect(self.configChanged)
-        repo.repositoryChanged.connect(self.repositoryChanged)
+        repoagent.configChanged.connect(self.configChanged)
+        repoagent.repositoryChanged.connect(self.repositoryChanged)
 
-    def done(self, ret):
-        self.repo.configChanged.disconnect(self.configChanged)
-        self.repo.repositoryChanged.disconnect(self.repositoryChanged)
-        super(ResolveDialog, self).done(ret)
+    @property
+    def repo(self):
+        return self._repoagent.rawRepo()
 
+    @pyqtSlot()
     def repositoryChanged(self):
         self.refresh()
 
@@ -207,7 +247,7 @@ class ResolveDialog(QDialog):
             cmd = cmdline + ['--repository', curroot, '--']
             for root, wfile in selected:
                 if root == curroot:
-                    cmd.append(os.path.join(root, wfile))
+                    cmd.append(os.path.normpath(os.path.join(root, wfile)))
             cmdlines.append(cmd)
             selected = [(r, w) for r, w in selected if r != curroot]
         if cmdlines:
@@ -234,7 +274,7 @@ class ResolveDialog(QDialog):
             qtlib.editfiles(self.repo, abspaths, parent=self)
 
     def getVdiffFiles(self, tree):
-        paths = self.getSelectedPaths(self.rtree)
+        paths = self.getSelectedPaths(tree)
         if not paths:
             return []
         files, sub = [], False
@@ -279,6 +319,27 @@ class ResolveDialog(QDialog):
             if dlg:
                 dlg.exec_()
 
+    def diffLocToAnc(self):
+        paths = self.getVdiffFiles(self.utree)
+        if paths:
+            opts = {}
+            opts['rev'] = ['ancestor(p1(),p2())..p1()']
+            opts['tool'] = self.tcombo.readValue()
+            dlg = visdiff.visualdiff(self.repo.ui, self.repo, paths, opts)
+            if dlg:
+                dlg.exec_()
+
+    def diffOthToAnc(self):
+        paths = self.getVdiffFiles(self.utree)
+        if paths:
+            opts = {}
+            opts['rev'] = ['ancestor(p1(),p2())..p2()']
+            opts['tool'] = self.tcombo.readValue()
+            dlg = visdiff.visualdiff(self.repo.ui, self.repo, paths, opts)
+            if dlg:
+                dlg.exec_()
+
+    @pyqtSlot()
     def configChanged(self):
         'repository has detected a change to config files'
         self.tcombo.reset()
@@ -293,9 +354,12 @@ class ResolveDialog(QDialog):
             else:
                 r.append((root, path))
         paths = self.getSelectedPaths(self.utree)
+        oldmodel = self.utree.model()
         self.utree.setModel(PathsModel(u, self))
         self.utree.resizeColumnToContents(0)
         self.utree.resizeColumnToContents(1)
+        if oldmodel:
+            oldmodel.setParent(None)  # gc-ed
 
         model = self.utree.model()
         smodel = self.utree.selectionModel()
@@ -311,13 +375,18 @@ class ResolveDialog(QDialog):
             enable = self.utree.selectionModel().hasSelection()
             for b in self.ubuttons:
                 b.setEnabled(enable)
+            for c in self.umenuitems:
+                c.setEnabled(enable)
         smodel.selectionChanged.connect(uchanged)
         uchanged(None, None)
 
         paths = self.getSelectedPaths(self.rtree)
+        oldmodel = self.rtree.model()
         self.rtree.setModel(PathsModel(r, self))
         self.rtree.resizeColumnToContents(0)
         self.rtree.resizeColumnToContents(1)
+        if oldmodel:
+            oldmodel.setParent(None)  # gc-ed
 
         model = self.rtree.model()
         smodel = self.rtree.selectionModel()
@@ -332,9 +401,13 @@ class ResolveDialog(QDialog):
             enable = self.rtree.selectionModel().hasSelection()
             for b in self.rbuttons:
                 b.setEnabled(enable)
+            for c in self.rmenuitems:
+                c.setEnabled(enable)
             merge = len(self.repo.parents()) > 1
             for b in self.rmbuttons:
                 b.setEnabled(enable and merge)
+            for c in self.rmmenuitems:
+                c.setEnabled(enable and merge)
         smodel.selectionChanged.connect(rchanged)
         rchanged(None, None)
 
@@ -349,7 +422,7 @@ class ResolveDialog(QDialog):
     def reject(self):
         s = QSettings()
         s.setValue('resolve/geom', self.saveGeometry())
-        if len(self.utree.model()):
+        if self.utree.model().rowCount() > 0:
             main = _('Exit without finishing resolve?')
             text = _('Unresolved conflicts remain. Are you sure?')
             labels = ((QMessageBox.Yes, _('E&xit')),
@@ -359,6 +432,20 @@ class ResolveDialog(QDialog):
                 return
         super(ResolveDialog, self).reject()
 
+    @pyqtSlot(QPoint)
+    def utreeMenuRequested(self, point):
+        self.utreecmenu.exec_(self.utree.viewport().mapToGlobal(point))
+
+    @pyqtSlot(QPoint)
+    def rtreeMenuRequested(self, point):
+        self.rtreecmenu.exec_(self.rtree.viewport().mapToGlobal(point))
+
+    def utreeDoubleClicked(self):
+        if self.repo.ui.configbool('tortoisehg', 'autoresolve'):
+            self.merge()
+        else:
+            self.merge('internal:merge')
+
 class PathsTree(QTreeView):
     def __init__(self, repo, parent):
         QTreeView.__init__(self, parent)
@@ -402,20 +489,17 @@ class PathsModel(QAbstractTableModel):
             name, ext = os.path.splitext(path)
             self.rows.append([path, ext, root])
 
-    def __len__(self):
-        return len(self.rows)
-
-    def rowCount(self, parent):
+    def rowCount(self, parent=QModelIndex()):
         if parent.isValid():
             return 0 # no child
         return len(self.rows)
 
-    def columnCount(self, parent):
+    def columnCount(self, parent=QModelIndex()):
         if parent.isValid():
             return 0 # no child
         return len(self.headers)
 
-    def data(self, index, role):
+    def data(self, index, role=Qt.DisplayRole):
         if not index.isValid():
             return QVariant()
         if role == Qt.DisplayRole:
@@ -423,7 +507,7 @@ class PathsModel(QAbstractTableModel):
             return QVariant(hglib.tounicode(data))
         return QVariant()
 
-    def headerData(self, col, orientation, role):
+    def headerData(self, col, orientation, role=Qt.DisplayRole):
         if role != Qt.DisplayRole or orientation != Qt.Horizontal:
             return QVariant()
         else:
@@ -464,8 +548,3 @@ class ToolsCombo(QComboBox):
                 return hglib.fromunicode(text)
         else:
             return None
-
-def run(ui, *pats, **opts):
-    from tortoisehg.util import paths
-    repo = thgrepo.repository(ui, path=paths.find_root())
-    return ResolveDialog(repo, None)
