@@ -28,9 +28,12 @@ class RevDetailsWidget(QWidget, qtlib.TaskWidget):
     updateToRevision = pyqtSignal(int)
     runCustomCommandRequested = pyqtSignal(str, list)
 
-    def __init__(self, repo, parent, rev=None):
+    def __init__(self, repoagent, parent, rev=None):
         QWidget.__init__(self, parent)
 
+        self._repoagent = repoagent
+        repo = repoagent.rawRepo()
+        # TODO: replace by repoagent if setRepo(bundlerepo) can be removed
         self.repo = repo
         self.ctx = repo[rev]
         self.splitternames = []
@@ -40,7 +43,7 @@ class RevDetailsWidget(QWidget, qtlib.TaskWidget):
         self.setupModels()
 
         self._deschtmlize = qtlib.descriptionhtmlizer(repo.ui)
-        repo.configChanged.connect(self._updatedeschtmlizer)
+        repoagent.configChanged.connect(self._updatedeschtmlizer)
 
     def setRepo(self, repo):
         self.repo = repo
@@ -88,7 +91,6 @@ class RevDetailsWidget(QWidget, qtlib.TaskWidget):
         self.filelisttbar = QToolBar(_('File List Toolbar'))
         self.filelisttbar.setIconSize(QSize(16,16))
         self.filelist = HgFileListView(self.repo, self, True)
-        self.filelist.linkActivated.connect(self.linkActivated)
         self.filelist.setContextMenuPolicy(Qt.CustomContextMenu)
         self.filelist.customContextMenuRequested.connect(self.menuRequest)
         self.filelist.doubleClicked.connect(self.onDoubleClick)
@@ -143,23 +145,23 @@ class RevDetailsWidget(QWidget, qtlib.TaskWidget):
                                     lineWrapMode=QTextEdit.NoWrap,
                                     openLinks=False)
         self.message.minimumSizeHint = lambda: QSize(0, 25)
-        self.message.anchorClicked.connect(
-            lambda url: self.linkActivated.emit(url.toString()))
+        self.message.anchorClicked.connect(self._forwardAnchorClicked)
 
         sp = SP(SP.Expanding, SP.Expanding)
         sp.setHorizontalStretch(0)
-        sp.setVerticalStretch(2)
+        sp.setVerticalStretch(0)
         sp.setHeightForWidth(self.message.sizePolicy().hasHeightForWidth())
         self.message.setSizePolicy(sp)
         self.message.setMinimumSize(QSize(0, 0))
+        self.message.sizeHint = lambda: QSize(0, 100)
         f = qtlib.getfont('fontcomment')
         self.message.setFont(f.font())
         f.changed.connect(self.forwardFont)
 
-        self.fileview = HgFileView(self.repo, self.messagesplitter)
+        self.fileview = HgFileView(self._repoagent, self.messagesplitter)
         sp = SP(SP.Expanding, SP.Expanding)
         sp.setHorizontalStretch(0)
-        sp.setVerticalStretch(5)
+        sp.setVerticalStretch(1)
         sp.setHeightForWidth(self.fileview.sizePolicy().hasHeightForWidth())
         self.fileview.setSizePolicy(sp)
         self.fileview.setMinimumSize(QSize(0, 0))
@@ -198,7 +200,7 @@ class RevDetailsWidget(QWidget, qtlib.TaskWidget):
     def createActions(self):
         self.actionUpdate = a = self.filelisttbar.addAction(
             qtlib.geticon('hg-update'), _('Update to this revision'))
-        a.triggered.connect(lambda: self.updateToRevision.emit(self.ctx.rev()))
+        a.triggered.connect(self._emitUpdateToRevision)
         self.filelisttbar.addSeparator()
         self.actionShowAllMerge = QAction(_('Show All'), self)
         self.actionShowAllMerge.setToolTip(
@@ -272,6 +274,14 @@ class RevDetailsWidget(QWidget, qtlib.TaskWidget):
             rev = 'tip'
         self.onRevisionSelected(rev)
 
+    @pyqtSlot(QUrl)
+    def _forwardAnchorClicked(self, url):
+        self.linkActivated.emit(url.toString())
+
+    @pyqtSlot()
+    def _emitUpdateToRevision(self):
+        self.updateToRevision.emit(self.ctx.rev())
+
     #@pyqtSlot(QModelIndex)
     def onDoubleClick(self, index):
         model = self.filelist.model()
@@ -335,11 +345,10 @@ class RevDetailsWidget(QWidget, qtlib.TaskWidget):
 class RevDetailsDialog(QDialog):
     'Standalone revision details tool, a wrapper for RevDetailsWidget'
 
-    def __init__(self, repo, rev='.', parent=None):
+    def __init__(self, repoagent, rev='.', parent=None):
         QDialog.__init__(self, parent)
         self.setWindowFlags(Qt.Window)
         self.setWindowIcon(qtlib.geticon('hg-log'))
-        self.repoviewer = None
 
         layout = QVBoxLayout()
         layout.setMargin(0)
@@ -349,7 +358,7 @@ class RevDetailsDialog(QDialog):
         toplayout.setContentsMargins(5, 5, 5, 0)
         layout.addLayout(toplayout)
 
-        revdetails = RevDetailsWidget(repo, parent, rev=rev)
+        revdetails = RevDetailsWidget(repoagent, parent, rev=rev)
         toplayout.addWidget(revdetails, 1)
 
         self.statusbar = cmdui.ThgStatusBar(self)
@@ -361,7 +370,7 @@ class RevDetailsDialog(QDialog):
         s = QSettings()
         self.restoreGeometry(s.value('revdetails/geom').toByteArray())
         revdetails.loadSettings(s)
-        repo.repositoryChanged.connect(self.refresh)
+        repoagent.repositoryChanged.connect(self.refresh)
 
         self.revdetails = revdetails
         self.setRev(rev)
@@ -389,15 +398,9 @@ class RevDetailsDialog(QDialog):
             except ValueError:
                 linkpath = linktarget
                 rev = None
-            if self.repoviewer is None:
-                # prevent recursive import
-                from workbench import Workbench
-                self.repoviewer = Workbench()
-            self.repoviewer.show()
-            self.repoviewer.activateWindow()
-            self.repoviewer.raise_()
-            self.repoviewer.showRepo(hglib.tounicode(linkpath))
-            self.repoviewer.goto(linkpath, rev)
+            # TODO: implement by using signal-slot if possible
+            from tortoisehg.hgqt import run
+            run.qtrun.showRepoInWorkbench(hglib.tounicode(linkpath), rev)
 
     @pyqtSlot()
     def refresh(self):
@@ -407,19 +410,11 @@ class RevDetailsDialog(QDialog):
         else:
             hash = self.revdetails.ctx.hex()[:12]
             revstr = '@%s: %s' % (str(revnum), hash)
-        self.setWindowTitle(_('%s - Revision Details (%s)') % (self.revdetails.repo.displayname, revstr))
+        self.setWindowTitle(_('%s - Revision Details (%s)')
+                            % (self.revdetails.repo.displayname, revstr))
         self.revdetails.reload()
 
     def done(self, ret):
         s = QSettings()
         s.setValue('revdetails/geom', self.saveGeometry())
         super(RevDetailsDialog, self).done(ret)
-
-def run(ui, *pats, **opts):
-    from tortoisehg.util import paths
-    from tortoisehg.hgqt import thgrepo
-    repo = thgrepo.repository(ui, path=paths.find_root())
-    pats = hglib.canonpaths(pats)
-    os.chdir(repo.root)
-    rev = opts.get('rev', '.')
-    return RevDetailsDialog(repo, rev=rev)
