@@ -35,10 +35,11 @@ console_commands = 'help thgstatus version'
 nonrepo_commands = '''userconfig shellconfig clone init debugbugreport
 about help version thgstatus serve rejects log'''
 
-def dispatch(args):
+def dispatch(args, u=None):
     """run the command specified in args"""
     try:
-        u = uimod.ui()
+        if u is None:
+            u = uimod.ui()
         if '--traceback' in args:
             u.setconfig('ui', 'traceback', 'on')
         if '--debugger' in args:
@@ -48,8 +49,8 @@ def dispatch(args):
         return _runcatch(u, args)
     except error.ParseError, e:
         qtapp.earlyExceptionMsgBox(e)
-    except SystemExit:
-        pass
+    except SystemExit, e:
+        return e.code
     except Exception, e:
         if '--debugger' in args:
             pdb.post_mortem(sys.exc_info()[2])
@@ -57,6 +58,7 @@ def dispatch(args):
         return -1
     except KeyboardInterrupt:
         print _('\nCaught keyboard interrupt, aborting.\n')
+        return -1
 
 origwdir = os.getcwd()
 def portable_fork(ui, opts):
@@ -225,20 +227,20 @@ def _runcatch(ui, args):
         finally:
             ui.flush()
     except error.AmbiguousCommand, inst:
-        ui.status(_("thg: command '%s' is ambiguous:\n    %s\n") %
+        ui.warn(_("thg: command '%s' is ambiguous:\n    %s\n") %
                 (inst.args[0], " ".join(inst.args[1])))
     except error.UnknownCommand, inst:
-        ui.status(_("thg: unknown command '%s'\n") % inst.args[0])
+        ui.warn(_("thg: unknown command '%s'\n") % inst.args[0])
         help_(ui, 'shortlist')
     except error.CommandError, inst:
         if inst.args[0]:
-            ui.status(_("thg %s: %s\n") % (inst.args[0], inst.args[1]))
+            ui.warn(_("thg %s: %s\n") % (inst.args[0], inst.args[1]))
             help_(ui, inst.args[0])
         else:
-            ui.status(_("thg: %s\n") % inst.args[1])
+            ui.warn(_("thg: %s\n") % inst.args[1])
             help_(ui, 'shortlist')
     except error.RepoError, inst:
-        ui.status(_("abort: %s!\n") % inst)
+        ui.warn(_("abort: %s!\n") % inst)
 
     return -1
 
@@ -383,8 +385,17 @@ def _filelog(ui, repoagent, *pats, **opts):
     if len(pats) != 1:
         raise util.Abort(_('requires a single filename'))
     filename = hglib.canonpaths(pats)[0]
-    repo = repoagent.rawRepo()
-    return filedialogs.FileLogDialog(repo, filename)
+    return filedialogs.FileLogDialog(repoagent, filename)
+
+def _formatfilerevset(pats):
+    q = []
+    for f in pats:
+        pat = hglib.canonpaths([f])[0]
+        if os.path.isdir(f):
+            q.append('file("%s/**")' % pat)
+        elif os.path.isfile(f):
+            q.append('file("%s")' % pat)
+    return ' or '.join(q)
 
 def _workbench(ui, *pats, **opts):
     root = opts.get('root') or paths.find_root()
@@ -401,14 +412,8 @@ def _workbench(ui, *pats, **opts):
             w.showRepo(root)
 
         if pats:
-            q = []
-            for f in pats:
-                pat = hglib.canonpaths([f])[0]
-                if os.path.isdir(f):
-                    q.append('file("%s/**")' % pat)
-                elif os.path.isfile(f):
-                    q.append('file("%s")' % pat)
-            w.setRevsetFilter(root, ' or '.join(q))
+            q = _formatfilerevset(pats)
+            w.setRevsetFilter(root, hglib.tounicode(q))
     if w.repoTabsWidget.count() <= 0:
         w.reporegistry.setVisible(True)
     return w
@@ -455,9 +460,8 @@ def annotate(ui, repoagent, *pats, **opts):
 def archive(ui, repoagent, *pats, **opts):
     """archive dialog"""
     from tortoisehg.hgqt import archive as archivemod
-    repo = repoagent.rawRepo()
     rev = opts.get('rev')
-    return archivemod.ArchiveDialog(repo, rev)
+    return archivemod.ArchiveDialog(repoagent, rev)
 
 @command('^backout',
     [('', 'merge', None, _('merge with old dirstate parent after backout')),
@@ -467,21 +471,19 @@ def archive(ui, repoagent, *pats, **opts):
 def backout(ui, repoagent, *pats, **opts):
     """backout tool"""
     from tortoisehg.hgqt import backout as backoutmod
-    repo = repoagent.rawRepo()
     if opts.get('rev'):
         rev = opts.get('rev')
     elif len(pats) == 1:
         rev = pats[0]
     else:
         rev = 'tip'
-    return backoutmod.BackoutDialog(rev, repo, None)
+    return backoutmod.BackoutDialog(repoagent, rev)
 
 @command('^bisect', [], _('thg bisect'))
 def bisect(ui, repoagent, *pats, **opts):
     """bisect dialog"""
     from tortoisehg.hgqt import bisect as bisectmod
-    repo = repoagent.rawRepo()
-    return bisectmod.BisectDialog(repo, opts)
+    return bisectmod.BisectDialog(repoagent, opts)
 
 @command('bookmarks|bookmark',
     [('r', 'rev', '', _('revision'))],
@@ -493,7 +495,7 @@ def bookmark(ui, repoagent, *names, **opts):
     rev = scmutil.revsingle(repo, opts.get('rev')).rev()
     if len(names) > 1:
         raise util.Abort(_('only one new bookmark name allowed'))
-    dlg = bookmarkmod.BookmarkDialog(repo, rev)
+    dlg = bookmarkmod.BookmarkDialog(repoagent, rev)
     if names:
         dlg.setBookmarkName(hglib.tounicode(names[0]))
     return dlg
@@ -523,7 +525,7 @@ def commit(ui, repoagent, *pats, **opts):
     repo = repoagent.rawRepo()
     pats = hglib.canonpaths(pats)
     os.chdir(repo.root)
-    return commitmod.CommitDialog(repo, pats, opts)
+    return commitmod.CommitDialog(repoagent, pats, opts)
 
 @command('debugbugreport', [], _('thg debugbugreport [TEXT]'))
 def debugbugreport(ui, *pats, **opts):
@@ -532,13 +534,13 @@ def debugbugreport(ui, *pats, **opts):
 
 @command('drag_copy', [], _('thg drag_copy SOURCE... DEST'))
 def drag_copy(ui, repoagent, *pats, **opts):
-    """Copy the selected files to the desired directory"""
+    """copy the selected files to the desired directory"""
     opts.update(alias='copy', headless=True)
     return quickop.run(ui, repoagent, *pats, **opts)
 
 @command('drag_move', [], _('thg drag_move SOURCE... DEST'))
 def drag_move(ui, repoagent, *pats, **opts):
-    """Move the selected files to the desired directory"""
+    """move the selected files to the desired directory"""
     opts.update(alias='move', headless=True)
     return quickop.run(ui, repoagent, *pats, **opts)
 
@@ -548,13 +550,15 @@ def drag_move(ui, repoagent, *pats, **opts):
 def email(ui, repoagent, *revs, **opts):
     """send changesets by email"""
     from tortoisehg.hgqt import hgemail
-    repo = repoagent.rawRepo()
     # TODO: same options as patchbomb
     if opts.get('rev'):
         if revs:
             raise util.Abort(_('use only one form to specify the revision'))
         revs = opts.get('rev')
-    return hgemail.EmailDialog(repo, revs)
+
+    repo = repoagent.rawRepo()
+    revs = scmutil.revrange(repo, revs)
+    return hgemail.EmailDialog(repoagent, revs)
 
 @command('forget', [], _('thg forget [FILE]...'))
 def forget(ui, repoagent, *pats, **opts):
@@ -572,7 +576,7 @@ def graft(ui, repoagent, *revs, **opts):
     revs.extend(opts['rev'])
     if not os.path.exists(repo.join('graftstate')) and not revs:
         raise util.Abort(_('You must provide revisions to graft'))
-    return graftmod.GraftDialog(repo, None, source=revs)
+    return graftmod.GraftDialog(repoagent, None, source=revs)
 
 @command('^grep|search',
     [('i', 'ignorecase', False, _('ignore case during search'))],
@@ -580,16 +584,14 @@ def graft(ui, repoagent, *revs, **opts):
 def grep(ui, repoagent, *pats, **opts):
     """grep/search dialog"""
     from tortoisehg.hgqt import grep as grepmod
-    repo = repoagent.rawRepo()
     upats = [hglib.tounicode(p) for p in pats]
-    return grepmod.SearchDialog(upats, repo, **opts)
+    return grepmod.SearchDialog(repoagent, upats, **opts)
 
 @command('^guess', [], _('thg guess'))
 def guess(ui, repoagent, *pats, **opts):
     """guess previous renames or copies"""
     from tortoisehg.hgqt import guess as guessmod
-    repo = repoagent.rawRepo()
-    return guessmod.DetectRenameDialog(repo, None, *pats)
+    return guessmod.DetectRenameDialog(repoagent, None, *pats)
 
 ### help management, adapted from mercurial.commands.help_()
 @command('help', [], _('thg help [COMMAND]'))
@@ -773,10 +775,9 @@ def help_(ui, name=None, with_version=False, **opts):
 def hgignore(ui, repoagent, *pats, **opts):
     """ignore filter editor"""
     from tortoisehg.hgqt import hgignore as hgignoremod
-    repo = repoagent.rawRepo()
     if pats and pats[0].endswith('.hgignore'):
         pats = []
-    return hgignoremod.HgignoreDialog(repo, None, *pats)
+    return hgignoremod.HgignoreDialog(repoagent, None, *pats)
 
 @command('import',
     [('', 'mq', False, _('import to the patch queue (MQ)'))],
@@ -784,8 +785,7 @@ def hgignore(ui, repoagent, *pats, **opts):
 def import_(ui, repoagent, *pats, **opts):
     """import an ordered set of patches"""
     from tortoisehg.hgqt import thgimport
-    repo = repoagent.rawRepo()
-    dlg = thgimport.ImportDialog(repo, None, **opts)
+    dlg = thgimport.ImportDialog(repoagent, None, **opts)
     dlg.setfilepaths(pats)
     return dlg
 
@@ -817,7 +817,7 @@ def log(ui, *pats, **opts):
     if singleworkbenchmode:
         newworkbench = opts.get('newworkbench')
         if root and not newworkbench:
-            if qtapp.connectToExistingWorkbench(root):
+            if qtapp.connectToExistingWorkbench(root, _formatfilerevset(pats)):
                 # The were able to connect to an existing workbench server, and
                 # it confirmed that it has opened the selected repo for us
                 sys.exit(0)
@@ -844,7 +844,7 @@ def manifest(ui, repoagent, *pats, **opts):
     from tortoisehg.hgqt import manifestdialog
     repo = repoagent.rawRepo()
     rev = scmutil.revsingle(repo, opts.get('rev')).rev()
-    dlg = manifestdialog.ManifestDialog(repo, rev)
+    dlg = manifestdialog.ManifestDialog(repoagent, rev)
     if pats:
         path = hglib.canonpaths(pats)[0]
         if opts.get('line'):
@@ -865,20 +865,12 @@ def manifest(ui, repoagent, *pats, **opts):
 def merge(ui, repoagent, *pats, **opts):
     """merge wizard"""
     from tortoisehg.hgqt import merge as mergemod
-    repo = repoagent.rawRepo()
     rev = opts.get('rev') or None
     if not rev and len(pats):
         rev = pats[0]
     if not rev:
         raise util.Abort(_('Merge revision not specified or not found'))
-    return mergemod.MergeDialog(rev, repo, None)
-
-@command('mq', [], _('thg mq'))
-def mq(ui, repoagent, *pats, **opts):
-    """Mercurial Queue tool"""
-    from tortoisehg.hgqt import mq as mqmod
-    repo = repoagent.rawRepo()
-    return mqmod.MQWidget(repo, None, **opts)
+    return mergemod.MergeDialog(repoagent, rev)
 
 @command('postreview',
     [('r', 'rev', [], _('a revision to post'))],
@@ -892,32 +884,13 @@ def postreview(ui, repoagent, *pats, **opts):
         revs = pats[0]
     if not revs:
         raise util.Abort(_('no revisions specified'))
-    return postreviewmod.PostReviewDialog(repo.ui, repo, revs)
+    return postreviewmod.PostReviewDialog(repo.ui, repoagent, revs)
 
 @command('^purge', [], _('thg purge'))
 def purge(ui, repoagent, *pats, **opts):
     """purge unknown and/or ignore files from repository"""
     from tortoisehg.hgqt import purge as purgemod
-    repo = repoagent.rawRepo()
-    return purgemod.PurgeDialog(repo, None)
-
-@command('^qqueue', [], _('thg qqueue'))
-def qqueue(ui, repoagent, *pats, **opts):
-    """manage multiple MQ patch queues"""
-    from tortoisehg.hgqt import qqueue as qqueuemod
-    repo = repoagent.rawRepo()
-    if not hasattr(repo, 'mq'):
-        raise util.Abort(_('Please enable the MQ extension first.'))
-    return qqueuemod.QQueueDialog(repo)
-
-@command('^qreorder', [], _('thg qreorder'))
-def qreorder(ui, repoagent, *pats, **opts):
-    """Reorder unapplied MQ patches"""
-    from tortoisehg.hgqt import qreorder as qreordermod
-    repo = repoagent.rawRepo()
-    if not hasattr(repo, 'mq'):
-        raise util.Abort(_('Please enable the MQ extension first.'))
-    return qreordermod.QReorderDialog(repo)
+    return purgemod.PurgeDialog(repoagent)
 
 @command('^rebase',
     [('', 'keep', False, _('keep original changesets')),
@@ -937,18 +910,18 @@ def rebase(ui, repoagent, *pats, **opts):
                                            'progress')))
     elif not opts['source'] or not opts['dest']:
         raise util.Abort(_('You must provide source and dest arguments'))
-    return rebasemod.RebaseDialog(repo, None, **opts)
+    return rebasemod.RebaseDialog(repoagent, None, **opts)
 
 @command('rejects', [], _('thg rejects [FILE]'))
 def rejects(ui, *pats, **opts):
-    """Manually resolve rejected patch chunks"""
+    """manually resolve rejected patch chunks"""
     from tortoisehg.hgqt import rejects as rejectsmod
     if len(pats) != 1:
         raise util.Abort(_('You must provide the path to a file'))
     path = pats[0]
     if path.endswith('.rej'):
         path = path[:-4]
-    return rejectsmod.RejectsDialog(path, None)
+    return rejectsmod.RejectsDialog(ui, path)
 
 @command('remove|rm', [], _('thg remove [FILE]...'))
 def remove(ui, repoagent, *pats, **opts):
@@ -959,9 +932,8 @@ def remove(ui, repoagent, *pats, **opts):
 def rename(ui, repoagent, *pats, **opts):
     """rename dialog"""
     from tortoisehg.hgqt import rename as renamemod
-    repo = repoagent.rawRepo()
     iscopy = (opts.get('alias') == 'copy')
-    return renamemod.RenameDialog(repo, pats, iscopy=iscopy)
+    return renamemod.RenameDialog(repoagent, pats, iscopy=iscopy)
 
 @command('^repoconfig',
     [('', 'focus', '', _('field to give initial focus'))],
@@ -975,8 +947,7 @@ def repoconfig(ui, repoagent, *pats, **opts):
 def resolve(ui, repoagent, *pats, **opts):
     """resolve dialog"""
     from tortoisehg.hgqt import resolve as resolvemod
-    repo = repoagent.rawRepo()
-    return resolvemod.ResolveDialog(repo)
+    return resolvemod.ResolveDialog(repoagent)
 
 @command('^revdetails',
     [('r', 'rev', '', _('the revision to show'))],
@@ -987,7 +958,7 @@ def revdetails(ui, repoagent, *pats, **opts):
     repo = repoagent.rawRepo()
     os.chdir(repo.root)
     rev = opts.get('rev', '.')
-    return revdetailsmod.RevDetailsDialog(repo, rev=rev)
+    return revdetailsmod.RevDetailsDialog(repoagent, rev=rev)
 
 @command('revert', [], _('thg revert [FILE]...'))
 def revert(ui, repoagent, *pats, **opts):
@@ -1000,13 +971,12 @@ def revert(ui, repoagent, *pats, **opts):
 def rupdate(ui, repoagent, *pats, **opts):
     """update a remote repository"""
     from tortoisehg.hgqt import rupdate as rupdatemod
-    repo = repoagent.rawRepo()
     rev = None
     if opts.get('rev'):
         rev = opts.get('rev')
     elif len(pats) == 1:
         rev = pats[0]
-    return rupdatemod.rUpdateDialog(repo, rev, None, opts)
+    return rupdatemod.rUpdateDialog(repoagent, rev, None, opts)
 
 @command('^serve',
     [('', 'web-conf', '', _('name of the hgweb config file (serve more than '
@@ -1028,10 +998,28 @@ if os.name == 'nt':
 
 @command('shelve|unshelve', [], _('thg shelve'))
 def shelve(ui, repoagent, *pats, **opts):
-    """Move changes between working directory and patches"""
+    """move changes between working directory and patches"""
     from tortoisehg.hgqt import shelve as shelvemod
+    return shelvemod.ShelveDialog(repoagent)
+
+@command('^sign',
+    [('f', 'force', None, _('sign even if the sigfile is modified')),
+     ('l', 'local', None, _('make the signature local')),
+     ('k', 'key', '', _('the key id to sign with')),
+     ('', 'no-commit', None, _('do not commit the sigfile after signing')),
+     ('m', 'message', '', _('use <text> as commit message'))],
+    _('thg sign [-f] [-l] [-k KEY] [-m TEXT] [REV]'))
+def sign(ui, repoagent, *pats, **opts):
+    """sign tool"""
+    from tortoisehg.hgqt import sign as signmod
     repo = repoagent.rawRepo()
-    return shelvemod.ShelveDialog(repo)
+    if 'gpg' not in repo.extensions():
+        raise util.Abort(_('Please enable the Gpg extension first.'))
+    kargs = {}
+    rev = len(pats) > 0 and pats[0] or None
+    if rev:
+        kargs['rev'] = rev
+    return signmod.SignDialog(repoagent, opts=opts, **kargs)
 
 @command('^status|st',
     [('c', 'clean', False, _('show files without changes')),
@@ -1043,7 +1031,7 @@ def status(ui, repoagent, *pats, **opts):
     repo = repoagent.rawRepo()
     pats = hglib.canonpaths(pats)
     os.chdir(repo.root)
-    return statusmod.StatusDialog(repo, pats, opts)
+    return statusmod.StatusDialog(repoagent, pats, opts)
 
 @command('^strip',
     [('f', 'force', None, _('discard uncommitted changes (no backup)')),
@@ -1053,20 +1041,18 @@ def status(ui, repoagent, *pats, **opts):
 def strip(ui, repoagent, *pats, **opts):
     """strip dialog"""
     from tortoisehg.hgqt import thgstrip
-    repo = repoagent.rawRepo()
     rev = None
     if opts.get('rev'):
         rev = opts.get('rev')
     elif len(pats) == 1:
         rev = pats[0]
-    return thgstrip.StripDialog(repo, rev=rev, opts=opts)
+    return thgstrip.StripDialog(repoagent, rev=rev, opts=opts)
 
 @command('^sync|synchronize', [], _('thg sync [PEER]'))
 def sync(ui, repoagent, *pats, **opts):
-    """Synchronize with other repositories"""
+    """synchronize with other repositories"""
     from tortoisehg.hgqt import sync as syncmod
-    repo = repoagent.rawRepo()
-    w = syncmod.SyncWidget(repo, None, **opts)
+    w = syncmod.SyncWidget(repoagent, None, **opts)
     if pats:
         w.setUrl(hglib.tounicode(pats[0]))
     return w
@@ -1081,7 +1067,6 @@ def sync(ui, repoagent, *pats, **opts):
 def tag(ui, repoagent, *pats, **opts):
     """tag tool"""
     from tortoisehg.hgqt import tag as tagmod
-    repo = repoagent.rawRepo()
     kargs = {}
     tag = len(pats) > 0 and pats[0] or None
     if tag:
@@ -1089,7 +1074,7 @@ def tag(ui, repoagent, *pats, **opts):
     rev = opts.get('rev')
     if rev:
         kargs['rev'] = rev
-    return tagmod.TagDialog(repo, opts=opts, **kargs)
+    return tagmod.TagDialog(repoagent, opts=opts, **kargs)
 
 @command('thgstatus',
     [('',  'delay', None, _('wait until the second ticks over')),
@@ -1111,13 +1096,12 @@ def thgstatus(ui, *pats, **opts):
 def update(ui, repoagent, *pats, **opts):
     """update/checkout tool"""
     from tortoisehg.hgqt import update as updatemod
-    repo = repoagent.rawRepo()
     rev = None
     if opts.get('rev'):
         rev = opts.get('rev')
     elif len(pats) == 1:
         rev = pats[0]
-    return updatemod.UpdateDialog(repo, rev, None, opts)
+    return updatemod.UpdateDialog(repoagent, rev, None, opts)
 
 @command('^userconfig',
     [('', 'focus', '', _('field to give initial focus'))],
