@@ -293,12 +293,3 @@ class ShellConfigWindow(QDialog):
             if list != itemlist:
                 list.setCurrentItem(None)
         self.update_states()
-
-def run(ui, *pats, **opts):
-    return ShellConfigWindow()
-
-if __name__ == "__main__":
-    app = QApplication(sys.argv)
-    form = ShellConfigWindow()
-    form.show()
-    app.exec_()
