@@ -8,7 +8,7 @@
 import os
 import time
 
-from mercurial import commands, error
+from mercurial import commands, error, util
 
 from tortoisehg.util import hglib
 from tortoisehg.util.patchctx import patchctx
@@ -20,16 +20,16 @@ from PyQt4.QtGui import *
 
 class ShelveDialog(QDialog):
 
-    finished = pyqtSignal(int)
     wdir = _('Working Directory')
 
-    def __init__(self, repo, parent):
+    def __init__(self, repoagent, parent=None):
         QDialog.__init__(self, parent)
         self.setWindowFlags(Qt.Window)
 
         self.setWindowIcon(qtlib.geticon('shelve'))
 
-        self.repo = repo
+        self._repoagent = repoagent
+        repo = repoagent.rawRepo()
         self.shelves = []
         self.patches = []
 
@@ -61,6 +61,7 @@ class ShelveDialog(QDialog):
         ahbox.setContentsMargins(2, 2, 2, 2)
         avbox.addLayout(ahbox)
         self.comboa = QComboBox(self)
+        self.comboa.setMinimumContentsLength(10)  # allow to cut long content
         self.comboa.currentIndexChanged.connect(self.comboAChanged)
         self.clearShelfButtonA = QPushButton(_('Clear'))
         self.clearShelfButtonA.setToolTip(_('Clear the current shelf file'))
@@ -72,7 +73,7 @@ class ShelveDialog(QDialog):
         ahbox.addWidget(self.clearShelfButtonA)
         ahbox.addWidget(self.delShelfButtonA)
 
-        self.browsea = chunks.ChunksWidget(repo, self, True)
+        self.browsea = chunks.ChunksWidget(self._repoagent, self, True)
         self.browsea.splitter.splitterMoved.connect(self.linkSplitters)
         self.browsea.linkActivated.connect(self.linkActivated)
         self.browsea.showMessage.connect(self.showMessage)
@@ -90,6 +91,7 @@ class ShelveDialog(QDialog):
         bhbox.setContentsMargins(2, 2, 2, 2)
         bvbox.addLayout(bhbox)
         self.combob = QComboBox(self)
+        self.combob.setMinimumContentsLength(10)  # allow to cut long content
         self.combob.currentIndexChanged.connect(self.comboBChanged)
         self.clearShelfButtonB = QPushButton(_('Clear'))
         self.clearShelfButtonB.setToolTip(_('Clear the current shelf file'))
@@ -101,7 +103,7 @@ class ShelveDialog(QDialog):
         bhbox.addWidget(self.clearShelfButtonB)
         bhbox.addWidget(self.delShelfButtonB)
 
-        self.browseb = chunks.ChunksWidget(repo, self, True)
+        self.browseb = chunks.ChunksWidget(self._repoagent, self, True)
         self.browseb.splitter.splitterMoved.connect(self.linkSplitters)
         self.browseb.linkActivated.connect(self.linkActivated)
         self.browseb.showMessage.connect(self.showMessage)
@@ -123,7 +125,7 @@ class ShelveDialog(QDialog):
         a.setIcon(qtlib.geticon('thg-shelve-move-right-file'))
         self.lefttbar.addAction(self.fileright)
         self.editfilea = a = QAction(_('Edit file'), self)
-        a.setIcon(qtlib.geticon('edit-find'))
+        a.setIcon(qtlib.geticon('edit-file'))
         self.lefttbar.addAction(self.editfilea)
         self.chunksright = a = QAction(_('Move selected chunks right'), self)
         self.chunksright.triggered.connect(self.moveChunksRight)
@@ -153,7 +155,7 @@ class ShelveDialog(QDialog):
         a.setIcon(qtlib.geticon('thg-shelve-move-left-chunks'))
         self.righttbar.addAction(self.chunksleft)
         self.editfileb = a = QAction(_('Edit file'), self)
-        a.setIcon(qtlib.geticon('edit-find'))
+        a.setIcon(qtlib.geticon('edit-file'))
         self.righttbar.addAction(self.editfileb)
         self.fileleft = a = QAction(_('Move selected file left'), self)
         self.fileleft.triggered.connect(self.moveFileLeft)
@@ -190,14 +192,14 @@ class ShelveDialog(QDialog):
                            'in .hg/Trashcan/'))
 
         self.refreshCombos()
-        repo.repositoryChanged.connect(self.refreshCombos)
+        repoagent.repositoryChanged.connect(self.refreshCombos)
 
         self.setWindowTitle(_('TortoiseHg Shelve - %s') % repo.displayname)
         self.restoreSettings()
 
-    def done(self, ret):
-        self.repo.repositoryChanged.disconnect(self.refreshCombos)
-        super(ShelveDialog, self).done(ret)
+    @property
+    def repo(self):
+        return self._repoagent.rawRepo()
 
     @pyqtSlot()
     def moveFileRight(self):
@@ -260,11 +262,16 @@ class ShelveDialog(QDialog):
             if not ok:
                 return
             shelve = hglib.fromunicode(name)
-            invalids = (':', '#', '/', '\\', '<', '>', '|')
+            invalids = (':', '#', '/', '\\')
             bads = [c for c in shelve if c in invalids]
             if bads:
                 qtlib.ErrorMsgBox(_('Bad filename'),
-                                  _('A shelf name cannot contain :#/\\<>|'))
+                                  _('A shelf name cannot contain %s')
+                                  % ''.join(bads))
+                return
+            badmsg = util.checkosfilename(shelve)
+            if badmsg:
+                qtlib.ErrorMsgBox(_('Bad filename'), hglib.tounicode(badmsg))
                 return
         try:
             fn = os.path.join('shelves', shelve)
@@ -429,6 +436,7 @@ class ShelveDialog(QDialog):
     def comboAChanged(self, index):
         if self.comboRefreshInProgress:
             return
+        assert index >= 0  # side A should always have "Working Directory"
         if index == 0:
             rev = None
             self.delShelfButtonA.setEnabled(False)
@@ -438,15 +446,27 @@ class ShelveDialog(QDialog):
             self.delShelfButtonA.setEnabled(index <= len(self.shelves))
             self.clearShelfButtonA.setEnabled(index <= len(self.shelves))
         self.browsea.setContext(self.repo.changectx(rev))
+        self._deselectSamePatch(self.combob)
 
     @pyqtSlot(int)
     def comboBChanged(self, index):
         if self.comboRefreshInProgress:
             return
         rev = self.currentPatchB()
-        self.delShelfButtonB.setEnabled(index < len(self.shelves))
-        self.clearShelfButtonB.setEnabled(index < len(self.shelves))
+        self.delShelfButtonB.setEnabled(0 <= index < len(self.shelves))
+        self.clearShelfButtonB.setEnabled(0 <= index < len(self.shelves))
         self.browseb.setContext(self.repo.changectx(rev))
+        self._deselectSamePatch(self.comboa)
+
+    def _deselectSamePatch(self, combo):
+        # if the same patch or shelve is selected by both sides, "move" action
+        # will corrupt patch content.
+        if self.currentPatchA() != self.currentPatchB():
+            return
+        if combo.count() > 1:
+            combo.setCurrentIndex((combo.currentIndex() + 1) % combo.count())
+        else:
+            combo.setCurrentIndex(-1)
 
     @pyqtSlot(int, int)
     def linkSplitters(self, pos, index):
@@ -487,12 +507,3 @@ class ShelveDialog(QDialog):
     def reject(self):
         self.storeSettings()
         super(ShelveDialog, self).reject()
-
-def run(ui, *pats, **opts):
-    if 'repo' in opts:
-        repo = opts['repo']
-    else:
-        from tortoisehg.util import paths
-        from tortoisehg.hgqt import thgrepo
-        repo = thgrepo.repository(ui, path=paths.find_root())
-    return ShelveDialog(repo, None)
