@@ -9,7 +9,7 @@ import os
 
 from mercurial import hg, util, error, context, merge, scmutil
 
-from tortoisehg.util import paths, hglib
+from tortoisehg.util import hglib
 from tortoisehg.hgqt.i18n import _
 from tortoisehg.hgqt import qtlib, wctxactions, visdiff, cmdui, fileview, thgrepo
 
@@ -50,14 +50,14 @@ class StatusWidget(QWidget):
     grepRequested = pyqtSignal(unicode, dict)
     runCustomCommandRequested = pyqtSignal(str, list)
 
-    def __init__(self, repo, pats, opts, parent=None, checkable=True,
-                 defcheck='MAR!S'):
+    def __init__(self, repoagent, pats, opts, parent=None, checkable=True,
+                 defcheck='commit'):
         QWidget.__init__(self, parent)
 
         self.opts = dict(modified=True, added=True, removed=True, deleted=True,
                          unknown=True, clean=False, ignored=False, subrepo=True)
         self.opts.update(opts)
-        self.repo = repo
+        self._repoagent = repoagent
         self.pats = pats
         self.checkable = checkable
         self.defcheck = defcheck
@@ -67,7 +67,6 @@ class StatusWidget(QWidget):
         self.refreshWctxLater = QTimer(self, interval=10, singleShot=True)
         self.refreshWctxLater.timeout.connect(self.refreshWctx)
         self.partials = {}
-        self.manualCheckAllUpdate = False
 
         # determine the user configured status colors
         # (in the future, we could support full rich-text tags)
@@ -124,7 +123,7 @@ class StatusWidget(QWidget):
             self.checkNoneTT = _('Uncheck all files')
             self.checkAllNoneBtn = QCheckBox()
             self.checkAllNoneBtn.setToolTip(self.checkAllTT)
-            self.checkAllNoneBtn.stateChanged.connect(self.checkAllNone)
+            self.checkAllNoneBtn.clicked.connect(self.checkAllNone)
 
         self.filelistToolbar = QToolBar(_('Status File List Toolbar'))
         self.filelistToolbar.setIconSize(QSize(16,16))
@@ -139,7 +138,8 @@ class StatusWidget(QWidget):
         self.filelistToolbar.addWidget(self.statusfilter)
         self.filelistToolbar.addSeparator()
         self.filelistToolbar.addWidget(self.refreshBtn)
-        self.actions = wctxactions.WctxActions(self.repo, self, checkable)
+        self.actions = wctxactions.WctxActions(self._repoagent, self, checkable)
+        self.actions.refreshNeeded.connect(self.refreshWctx)
         self.actions.runCustomCommandRequested.connect(
             self.runCustomCommandRequested)
         tv = WctxFileTree(self.repo, checkable=checkable)
@@ -177,7 +177,7 @@ class StatusWidget(QWidget):
         docf.setLayout(vbox)
         self.docf = docf
 
-        self.fileview = fileview.HgFileView(self.repo, self)
+        self.fileview = fileview.HgFileView(self._repoagent, self)
         self.fileview.showMessage.connect(self.showMessage)
         self.fileview.linkActivated.connect(self.linkActivated)
         self.fileview.fileDisplayed.connect(self.fileDisplayed)
@@ -192,9 +192,26 @@ class StatusWidget(QWidget):
         self.split = split
         self.diffvbox = vbox
 
+    @property
+    def repo(self):
+        return self._repoagent.rawRepo()
+
+    def __get_defcheck(self):
+        if self._defcheck is None:
+            return 'MAR!S'
+        return self._defcheck
+
+    def __set_defcheck(self, newdefcheck):
+        if newdefcheck.lower() == 'amend':
+            newdefcheck = 'MAS'
+        elif newdefcheck.lower() in ('commit', 'qnew', 'qrefresh'):
+            newdefcheck = 'MAR!S'
+        self._defcheck = newdefcheck
+
+    defcheck = property(__get_defcheck, __set_defcheck)
+
+    @pyqtSlot()
     def checkAllNone(self):
-        if self.manualCheckAllUpdate:
-            return
         state = self.checkAllNoneBtn.checkState()
         if state == Qt.Checked:
             self.checkAll()
@@ -226,6 +243,7 @@ class StatusWidget(QWidget):
         # selections, in order to simplify refresh.
         dels = []
         for file, oldchanges in self.partials.iteritems():
+            assert file in self.tv.model().checked
             if oldchanges.excludecount == 0:
                 self.tv.model().checked[file] = True
                 dels.append(file)
@@ -235,12 +253,13 @@ class StatusWidget(QWidget):
         for file in dels:
             del self.partials[file]
 
+        wfile = hglib.fromunicode(wfile)
         if changes is None:
             if wfile in self.partials:
                 del self.partials[wfile]
+                self.chunkSelectionChanged()
             return
 
-        wfile = hglib.fromunicode(wfile)
         if wfile in self.partials:
             # merge selection state from old hunk list to new hunk list
             oldhunks = self.partials[wfile].hunks
@@ -270,8 +289,7 @@ class StatusWidget(QWidget):
     @pyqtSlot(QPoint, object)
     def onMenuRequest(self, point, selected):
         menu = self.actions.makeMenu(selected)
-        if menu.exec_(point):
-            self.refreshWctx()
+        menu.exec_(point)
 
     def setPatchContext(self, pctx):
         if pctx != self.pctx:
@@ -307,7 +325,7 @@ class StatusWidget(QWidget):
         self.progress.emit(*cmdui.startProgress(_('Refresh'), _('status')))
         self.refthread = StatusThread(self.repo, self.pctx, self.pats, self.opts)
         self.refthread.finished.connect(self.reloadComplete)
-        self.refthread.showMessage.connect(self.showMessage)
+        self.refthread.showMessage.connect(self.reloadFailed)
         self.refthread.start()
 
     @pyqtSlot()
@@ -327,6 +345,11 @@ class StatusWidget(QWidget):
         if match:
             self.setFilter(match)
 
+    # better to handle error in reloadComplete in place of separate signal?
+    @pyqtSlot(QString)
+    def reloadFailed(self, msg):
+        qtlib.ErrorMsgBox(_('Failed to refresh'), msg, parent=self)
+
     def isRefreshingWctx(self):
         return bool(self.refthread)
 
@@ -362,6 +385,11 @@ class StatusWidget(QWidget):
         if self.checkable:
             self.updateCheckCount()
 
+        # remove non-existent file from partials table because model changed
+        for file in self.partials.keys():
+            if file not in tm.checked:
+                del self.partials[file]
+
         for col in (COL_PATH, COL_STATUS, COL_MERGE_STATE):
             w = self.tv.sizeHintForColumn(col)
             self.tv.setColumnWidth(col, w)
@@ -441,9 +469,8 @@ class StatusWidget(QWidget):
                 state = Qt.Checked
             else:
                 state = Qt.PartiallyChecked
-            self.manualCheckAllUpdate = True
+            self.checkAllNoneBtn.setTristate(state == Qt.PartiallyChecked)
             self.checkAllNoneBtn.setCheckState(state)
-            self.manualCheckAllUpdate = False
 
     @pyqtSlot(QString, bool)
     def checkToggled(self, wfile, checked):
@@ -747,6 +774,7 @@ class WctxModel(QAbstractTableModel):
 
     def check(self, files, state):
         for f in files:
+            assert f in checked
             self.checked[f] = state
             self.checkToggled.emit(f, state)
         self.layoutChanged.emit()
@@ -936,6 +964,7 @@ class WctxModel(QAbstractTableModel):
         self.reset()
 
     def getChecked(self):
+        assert len(self.checked) == len(self.unfiltered)
         return self.checked.copy()
 
 def statusMessage(status, mst, upath):
@@ -1026,15 +1055,16 @@ class StatusFilterButton(QToolButton):
 
 class StatusDialog(QDialog):
     'Standalone status browser'
-    def __init__(self, repo, pats, opts, parent=None):
+    def __init__(self, repoagent, pats, opts, parent=None):
         QDialog.__init__(self, parent)
         self.setWindowIcon(qtlib.geticon('hg-status'))
         layout = QVBoxLayout()
         layout.setContentsMargins(0, 0, 0, 0)
         self.setLayout(layout)
         toplayout = QVBoxLayout()
-        toplayout.setContentsMargins(10, 10, 10, 0);
-        self.stwidget = StatusWidget(repo, pats, opts, self, checkable=False)
+        toplayout.setContentsMargins(10, 10, 10, 0)
+        self.stwidget = StatusWidget(repoagent, pats, opts, self,
+                                     checkable=False)
         toplayout.addWidget(self.stwidget, 1)
         layout.addLayout(toplayout)
 
@@ -1045,6 +1075,9 @@ class StatusDialog(QDialog):
         self.stwidget.titleTextChanged.connect(self.setWindowTitle)
         self.stwidget.linkActivated.connect(self.linkActivated)
 
+        self._subdialogs = qtlib.DialogKeeper(StatusDialog._createSubDialog,
+                                              parent=self)
+
         self.setWindowTitle(self.stwidget.getTitle())
         self.setWindowFlags(Qt.Window)
         self.loadSettings()
@@ -1054,11 +1087,15 @@ class StatusDialog(QDialog):
         QTimer.singleShot(0, self.stwidget.refreshWctx)
 
     def linkActivated(self, link):
-        link = hglib.fromunicode(link)
+        link = unicode(link)
         if link.startswith('repo:'):
-            from tortoisehg.hgqt.run import qtrun
-            from tortoisehg.hgqt import commit
-            qtrun(commit.run, self.stwidget.repo.ui, root=link[len('repo:'):])
+            self._subdialogs.open(link[len('repo:'):])
+
+    def _createSubDialog(self, uroot):
+        # TODO: do not instantiate repo here
+        repo = thgrepo.repository(None, hglib.fromunicode(uroot))
+        repoagent = repo._pyqtobj
+        return StatusDialog(repoagent, [], {}, parent=self)
 
     def loadSettings(self):
         s = QSettings()
@@ -1081,9 +1118,3 @@ class StatusDialog(QDialog):
             return
         self.saveSettings()
         QDialog.reject(self)
-
-def run(ui, *pats, **opts):
-    repo = thgrepo.repository(ui, path=paths.find_root())
-    pats = hglib.canonpaths(pats)
-    os.chdir(repo.root)
-    return StatusDialog(repo, pats, opts)
